# AI Web Automation

## What is the AI Web Automation module?

The AI Web Automation module provides a comprehensive framework for automating web interactions using AI-powered browser automation with Playwright MCP (Model Context Protocol) integration. It enables site builders and developers to create, manage, and execute complex web automation scenarios through a user-friendly interface.

## Key Features

- **Visual Scenario Builder**: Create automation scenarios with a drag-and-drop interface
- **Extensible Step System**: Plugin-based architecture for automation steps
- **MCP Integration**: Seamless integration with Playwright MCP server for browser automation
- **Real-time Execution Monitoring**: Watch your automations execute in real-time
- **Variable System**: Use variables for dynamic content and data extraction
- **Results Storage**: Comprehensive logging and screenshot capture
- **Security Controls**: URL whitelisting/blacklisting and JavaScript execution controls

## Dependencies

- Drupal ^11
- PHP >= 8.3
- [AI module](https://www.drupal.org/project/ai) (for menu integration)
- Playwright MCP Server (external service)

## Installation

1. Install the module using Composer:
   ```bash
   composer require drupal/ai_web_automation
   ```

2. Enable the module:
   ```bash
   drush en ai_web_automation -y
   ```

3. Configure the MCP server connection at `/admin/config/ai/ai-web-automation`

## Configuration

### MCP Server Setup

Before using this module, you need to have a Playwright MCP server running. The MCP server handles the actual browser automation commands.

1. Navigate to **Configuration > AI > AI Web Automation**
2. Enter your MCP Server URL (e.g., `http://localhost:3000`)
3. Optionally configure an API key for authentication
4. Test the connection to ensure it's working

### Security Settings

- **URL Whitelist/Blacklist**: Control which URLs can be accessed during automation
- **JavaScript Execution**: Enable/disable custom JavaScript execution in scenarios
- **Browser Context**: Configure headless mode and viewport settings

## Usage

### Creating an Automation Scenario

1. Navigate to **Content > Automation Scenarios**
2. Click **Add automation scenario**
3. Add steps to your scenario:
   - **Navigate**: Go to a URL
   - **Click**: Click on elements
   - **Type**: Fill in forms
   - **Extract**: Extract data from pages
   - **Screenshot**: Capture screenshots
   - **Wait**: Wait for conditions or time
4. Configure variables for dynamic content
5. Save and execute your scenario

### Available Step Types

- **Navigation Steps**: Navigate, Scroll, Hover
- **Interaction Steps**: Click, Type, Select
- **Data Steps**: Extract content, Assert conditions
- **Utility Steps**: Wait, Screenshot, Custom JavaScript

### Executing Scenarios

Scenarios can be executed in multiple modes:
- **Normal**: Execute all steps sequentially
- **Dry Run**: Preview without actual execution
- **Step-by-Step**: Pause between steps for debugging

## Permissions

The module provides granular permissions:
- `administer ai web automation`: Full administrative access
- `create automation scenarios`: Create new scenarios
- `edit own/any automation scenarios`: Edit scenarios
- `delete own/any automation scenarios`: Delete scenarios
- `execute own/any automation scenarios`: Execute scenarios
- `view automation results`: View execution results

## API and Development

### Creating Custom Step Plugins

Create a new step type by extending the `AutomationStepBase` class:

```php
namespace Drupal\mymodule\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;

/**
 * @AutomationStep(
 *   id = "my_custom_step",
 *   label = @Translation("My Custom Step"),
 *   description = @Translation("Description of what this step does"),
 *   category = "custom",
 *   icon = "custom",
 *   mcp_tool = "playwright_custom"
 * )
 */
class MyCustomStep extends AutomationStepBase {
  // Implementation
}
```

### Programmatic Execution

Execute scenarios programmatically:

```php
$executor = \Drupal::service('ai_web_automation.scenario_executor');
$scenario = \Drupal::entityTypeManager()
  ->getStorage('automation_scenario')
  ->load($scenario_id);

$execution_id = $executor->execute($scenario, [
  'variables' => ['custom_var' => 'value'],
  'capture_screenshots' => TRUE,
]);
```

## Troubleshooting

### Common Issues

1. **MCP Connection Failed**: Ensure your MCP server is running and accessible
2. **Entity Schema Issues**: Run `drush cr` after installation
3. **Permission Denied**: Check user permissions for automation scenarios

### Debug Mode

Enable debug logging in the module settings to see detailed execution logs.

## Security Considerations

- Always use URL whitelisting in production environments
- Be cautious with JavaScript execution permissions
- Regularly review and audit automation scenarios
- Store sensitive data (passwords, API keys) using Drupal's Key module

## Roadmap

- Import/Export functionality for scenarios
- Scheduling system for automated execution
- Integration with Drupal Queue API
- Enhanced conditional logic
- Multi-step transactions with rollback

## Contributing

Contributions are welcome! Please submit issues and merge requests on the project page.

## Maintainers

- Chief Maintainer @sudofly

## License

This project is licensed under the GNU General Public License v2.0 or later.
