/**
 * @file
 * Real-time execution monitoring for AI Web Automation.
 */

(function ($, Drupal, drupalSettings) {
  'use strict';

  /**
   * Execution monitor behavior.
   */
  Drupal.behaviors.aiWebAutomationExecutionMonitor = {
    attach: function (context, settings) {
      const $container = $('.automation-execution-live', context).once('execution-monitor');
      
      if ($container.length) {
        const monitor = new ExecutionMonitor($container);
        monitor.start();
      }
    }
  };

  /**
   * ExecutionMonitor class for managing live execution.
   */
  class ExecutionMonitor {
    constructor($container) {
      this.$container = $container;
      this.executionId = $container.data('execution-id');
      this.scenarioId = $container.data('scenario-id');
      this.pollInterval = null;
      this.currentStep = 0;
      this.totalSteps = $('.execution-step', $container).length;
      
      this.initializeElements();
      this.bindEvents();
    }

    initializeElements() {
      this.$progressBar = $('.progress-bar__fill', this.$container);
      this.$currentStep = $('.current-step', this.$container);
      this.$statusMessage = $('.status-message', this.$container);
      this.$outputLog = $('.output-log', this.$container);
      this.$screenshot = $('.screenshot-container img', this.$container);
      this.$screenshotPlaceholder = $('.screenshot-placeholder', this.$container);
      this.$completionSection = $('.execution-complete', this.$container);
      this.$viewResultsLink = $('.view-results', this.$container);
    }

    bindEvents() {
      $('#cancel-execution', this.$container).on('click', this.cancelExecution.bind(this));
    }

    start() {
      this.updateStatus('Starting execution...');
      this.addLogEntry('Execution started at ' + new Date().toLocaleTimeString());
      
      // Start polling for status
      this.pollInterval = setInterval(this.pollStatus.bind(this), 1000);
      
      // Start execution
      this.startExecution();
    }

    startExecution() {
      const url = Drupal.url('automation-scenario/' + this.scenarioId + '/execute-step');
      
      $.ajax({
        url: url,
        method: 'POST',
        dataType: 'json',
        contentType: 'application/json',
        data: JSON.stringify({
          execution_id: this.executionId,
          step_index: 0,
          context: {
            variables: {},
            previous_results: []
          }
        }),
        success: this.handleStepComplete.bind(this),
        error: this.handleError.bind(this)
      });
    }

    pollStatus() {
      const url = Drupal.url('automation/execution/status');
      
      $.ajax({
        url: url,
        method: 'GET',
        dataType: 'json',
        data: {
          execution_id: this.executionId
        },
        success: (data) => {
          if (data.status === 'completed' || data.status === 'failed') {
            this.executionComplete(data);
          }
        }
      });
    }

    handleStepComplete(response) {
      const stepIndex = this.currentStep;
      const $step = $('.execution-step[data-step-index="' + stepIndex + '"]', this.$container);
      
      if (response.result.success) {
        $step.removeClass('active').addClass('completed');
        $('.step-status', $step).html('<span class="status-icon status-icon--success">✓</span>');
        this.addLogEntry('Step ' + (stepIndex + 1) + ' completed successfully');
      } else {
        $step.removeClass('active').addClass('failed');
        $('.step-status', $step).html('<span class="status-icon status-icon--failed">✗</span>');
        this.addLogEntry('Step ' + (stepIndex + 1) + ' failed: ' + response.result.error);
      }
      
      // Update step details
      if (response.result.data) {
        $('.step-details', $step).text(JSON.stringify(response.result.data));
      }
      
      // Update screenshot if available
      if (response.result.screenshot) {
        this.updateScreenshot(response.result.screenshot);
      }
      
      // Progress to next step
      this.currentStep++;
      this.updateProgress();
      
      if (response.next_step !== null && response.result.success) {
        // Execute next step
        setTimeout(() => {
          this.executeStep(response.next_step);
        }, 500);
      } else {
        // Execution complete
        this.executionComplete({
          status: response.result.success ? 'completed' : 'failed'
        });
      }
    }

    executeStep(stepIndex) {
      const $step = $('.execution-step[data-step-index="' + stepIndex + '"]', this.$container);
      $step.addClass('active');
      
      this.updateStatus('Executing step ' + (stepIndex + 1) + ' of ' + this.totalSteps);
      
      const url = Drupal.url('automation-scenario/' + this.scenarioId + '/execute-step');
      
      $.ajax({
        url: url,
        method: 'POST',
        dataType: 'json',
        contentType: 'application/json',
        data: JSON.stringify({
          execution_id: this.executionId,
          step_index: stepIndex,
          context: {
            variables: {},
            previous_results: []
          }
        }),
        success: this.handleStepComplete.bind(this),
        error: this.handleError.bind(this)
      });
    }

    updateProgress() {
      const percent = (this.currentStep / this.totalSteps) * 100;
      this.$progressBar.css('width', percent + '%');
      this.$currentStep.text(this.currentStep);
    }

    updateStatus(message) {
      this.$statusMessage.text(message);
    }

    addLogEntry(message) {
      const timestamp = new Date().toLocaleTimeString();
      const entry = '[' + timestamp + '] ' + message + '\n';
      this.$outputLog.append(entry);
      
      // Auto-scroll to bottom
      const container = this.$outputLog.parent()[0];
      container.scrollTop = container.scrollHeight;
    }

    updateScreenshot(fileId) {
      // In a real implementation, this would fetch the screenshot URL
      const url = Drupal.url('automation/screenshot/' + fileId);
      this.$screenshot.attr('src', url).show();
      this.$screenshotPlaceholder.hide();
    }

    handleError(xhr, status, error) {
      this.addLogEntry('Error: ' + error);
      this.updateStatus('Execution failed');
      
      clearInterval(this.pollInterval);
      
      this.$completionSection.show();
      $('.completion-message', this.$completionSection).text('Execution failed: ' + error);
    }

    executionComplete(data) {
      clearInterval(this.pollInterval);
      
      this.updateProgress();
      this.updateStatus('Execution ' + data.status);
      this.addLogEntry('Execution completed with status: ' + data.status);
      
      // Update view results link
      const resultsUrl = Drupal.url('automation-scenario/' + this.scenarioId + '/results/' + this.executionId);
      this.$viewResultsLink.attr('href', resultsUrl);
      
      this.$completionSection.show();
      $('.completion-message', this.$completionSection).text(
        data.status === 'completed' 
          ? 'Execution completed successfully!' 
          : 'Execution failed. Check the results for details.'
      );
    }

    cancelExecution() {
      if (!confirm(Drupal.t('Are you sure you want to cancel this execution?'))) {
        return;
      }
      
      clearInterval(this.pollInterval);
      
      const url = Drupal.url('automation/execution/cancel');
      
      $.ajax({
        url: url,
        method: 'POST',
        dataType: 'json',
        contentType: 'application/json',
        data: JSON.stringify({
          execution_id: this.executionId
        }),
        success: () => {
          this.updateStatus('Execution cancelled');
          this.addLogEntry('Execution cancelled by user');
          this.executionComplete({ status: 'cancelled' });
        }
      });
    }
  }

})(jQuery, Drupal, drupalSettings);