<?php

namespace Drupal\ai_web_automation;

use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for Automation Step plugins.
 */
abstract class AutomationStepBase extends PluginBase implements AutomationStepInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->pluginDefinition['description'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function getIcon() {
    return $this->pluginDefinition['icon'] ?? 'default';
  }

  /**
   * {@inheritdoc}
   */
  public function getCategory() {
    return $this->pluginDefinition['category'] ?? 'general';
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = $configuration + $this->defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Override in subclasses to provide validation.
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Override in subclasses to handle submission.
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = [];
    // Override in subclasses to provide validation.
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    return $this->t('Execute @label step', ['@label' => $this->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function supportsConditions() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function capturesScreenshot() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->pluginDefinition['weight'] ?? 0;
  }

  /**
   * Helper method to interpolate variables in a string.
   *
   * @param string $string
   *   The string containing variable placeholders.
   * @param array $variables
   *   The variables to interpolate.
   *
   * @return string
   *   The interpolated string.
   */
  protected function interpolateVariables($string, array $variables) {
    if (empty($variables)) {
      return $string;
    }

    // Replace {{variable}} placeholders.
    return preg_replace_callback('/\{\{(\w+)\}\}/', function ($matches) use ($variables) {
      $key = $matches[1];
      return isset($variables[$key]) ? $variables[$key] : $matches[0];
    }, $string);
  }

  /**
   * Helper method to measure execution time.
   *
   * @param callable $callback
   *   The callback to execute.
   *
   * @return array
   *   Array with 'result' and 'duration' keys.
   */
  protected function measureExecution(callable $callback) {
    $start = microtime(TRUE);
    $result = $callback();
    $duration = (microtime(TRUE) - $start) * 1000; // Convert to milliseconds.
    
    return [
      'result' => $result,
      'duration' => round($duration, 2),
    ];
  }

}