<?php

namespace Drupal\ai_web_automation;

use Drupal\Component\Plugin\ConfigurableInterface;
use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginFormInterface;

/**
 * Defines an interface for Automation Step plugins.
 */
interface AutomationStepInterface extends PluginInspectionInterface, ConfigurableInterface, PluginFormInterface, ContainerFactoryPluginInterface {

  /**
   * Returns the step label.
   *
   * @return string
   *   The step label.
   */
  public function label();

  /**
   * Returns the step description.
   *
   * @return string
   *   The step description.
   */
  public function description();

  /**
   * Returns the step icon.
   *
   * @return string
   *   The icon identifier (e.g., 'navigate', 'click', 'type').
   */
  public function getIcon();

  /**
   * Returns the step category.
   *
   * @return string
   *   The category (e.g., 'navigation', 'interaction', 'assertion').
   */
  public function getCategory();

  /**
   * Validates the step configuration.
   *
   * @return array
   *   An array of validation errors, or empty if valid.
   */
  public function validate();

  /**
   * Executes the automation step.
   *
   * @param array $context
   *   The execution context containing:
   *   - 'mcp_client': The MCP client service.
   *   - 'variables': Array of scenario variables.
   *   - 'previous_results': Results from previous steps.
   *   - 'session_id': The browser session ID.
   *
   * @return array
   *   The execution result containing:
   *   - 'success': Boolean indicating success.
   *   - 'data': Any data returned by the step.
   *   - 'error': Error message if failed.
   *   - 'screenshot': Screenshot file ID if captured.
   *   - 'duration': Execution time in milliseconds.
   */
  public function execute(array $context);

  /**
   * Returns a preview of what this step will do.
   *
   * @return string
   *   A human-readable preview of the step action.
   */
  public function preview();

  /**
   * Returns example configurations for this step type.
   *
   * @return array
   *   Array of example configurations.
   */
  public function getExamples();

  /**
   * Returns the MCP tool name for this step.
   *
   * @return string
   *   The MCP tool name (e.g., 'playwright_navigate').
   */
  public function getMcpTool();

  /**
   * Prepares MCP parameters from the step configuration.
   *
   * @param array $variables
   *   Variables for interpolation.
   *
   * @return array
   *   The parameters for the MCP tool.
   */
  public function prepareMcpParameters(array $variables = []);

  /**
   * Whether this step can be used in conditional logic.
   *
   * @return bool
   *   TRUE if the step can be used in conditions.
   */
  public function supportsConditions();

  /**
   * Whether this step captures a screenshot.
   *
   * @return bool
   *   TRUE if the step captures screenshots.
   */
  public function capturesScreenshot();

  /**
   * Gets the weight for ordering steps.
   *
   * @return int
   *   The weight value.
   */
  public function getWeight();

}