<?php

namespace Drupal\ai_web_automation;

use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * Provides the Automation Step plugin manager.
 */
class AutomationStepManager extends DefaultPluginManager {

  /**
   * Constructs a new AutomationStepManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct(
      'Plugin/AutomationStep',
      $namespaces,
      $module_handler,
      'Drupal\ai_web_automation\AutomationStepInterface',
      'Drupal\ai_web_automation\Annotation\AutomationStep'
    );

    $this->alterInfo('ai_web_automation_step_info');
    $this->setCacheBackend($cache_backend, 'ai_web_automation_step_plugins');
  }

  /**
   * Gets all available step definitions grouped by category.
   *
   * @return array
   *   Array of step definitions grouped by category.
   */
  public function getStepsByCategory() {
    $definitions = $this->getDefinitions();
    $grouped = [];
    
    foreach ($definitions as $id => $definition) {
      $category = $definition['category'] ?? 'general';
      $grouped[$category][$id] = $definition;
    }
    
    // Sort categories and steps within categories.
    ksort($grouped);
    foreach ($grouped as &$steps) {
      uasort($steps, function ($a, $b) {
        $weight_a = $a['weight'] ?? 0;
        $weight_b = $b['weight'] ?? 0;
        if ($weight_a == $weight_b) {
          return strcmp($a['label'], $b['label']);
        }
        return $weight_a <=> $weight_b;
      });
    }
    
    return $grouped;
  }

  /**
   * Gets step options for form select elements.
   *
   * @return array
   *   Array of options keyed by plugin ID.
   */
  public function getStepOptions() {
    $options = [];
    $definitions = $this->getDefinitions();
    
    foreach ($definitions as $id => $definition) {
      $options[$id] = $definition['label'];
    }
    
    asort($options);
    return $options;
  }

  /**
   * Creates a step instance with the given configuration.
   *
   * @param string $plugin_id
   *   The plugin ID.
   * @param array $configuration
   *   The plugin configuration.
   *
   * @return \Drupal\ai_web_automation\AutomationStepInterface
   *   The step plugin instance.
   */
  public function createStepInstance($plugin_id, array $configuration = []) {
    return $this->createInstance($plugin_id, $configuration);
  }

  /**
   * Validates a step configuration.
   *
   * @param string $plugin_id
   *   The plugin ID.
   * @param array $configuration
   *   The plugin configuration.
   *
   * @return array
   *   Array of validation errors, empty if valid.
   */
  public function validateStep($plugin_id, array $configuration) {
    try {
      $step = $this->createStepInstance($plugin_id, $configuration);
      return $step->validate();
    }
    catch (\Exception $e) {
      return [$e->getMessage()];
    }
  }

}