<?php

namespace Drupal\ai_web_automation\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Controller for automation-related pages.
 */
class AutomationController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs an AutomationController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Title callback for automation scenario pages.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   *
   * @return string
   *   The page title.
   */
  public function title(AutomationScenarioInterface $automation_scenario) {
    return $automation_scenario->getName();
  }

  /**
   * Displays the automation dashboard.
   *
   * @return array
   *   Render array for the dashboard.
   */
  public function dashboard() {
    $build = [
      '#theme' => 'automation_dashboard',
      '#attached' => [
        'library' => ['ai_web_automation/dashboard'],
      ],
    ];

    // Get recent scenarios.
    $scenario_storage = $this->entityTypeManager->getStorage('automation_scenario');
    $recent_scenarios = $scenario_storage->getQuery()
      ->condition('status', 1)
      ->sort('changed', 'DESC')
      ->range(0, 5)
      ->accessCheck(TRUE)
      ->execute();

    if ($recent_scenarios) {
      $scenarios = $scenario_storage->loadMultiple($recent_scenarios);
      $build['#recent_scenarios'] = $scenarios;
    }

    // Get execution statistics.
    $results_storage = \Drupal::service('ai_web_automation.results_storage');
    $stats = $this->getExecutionStatistics();
    $build['#statistics'] = $stats;

    return $build;
  }

  /**
   * Displays the live execution view.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   *
   * @return array
   *   Render array for the live execution page.
   */
  public function executeLive(AutomationScenarioInterface $automation_scenario) {
    // Get execution options from session.
    $session = \Drupal::request()->getSession();
    $options = $session->get('ai_web_automation_execution_options', []);
    
    // Start execution.
    $executor = \Drupal::service('ai_web_automation.scenario_executor');
    $execution_id = uniqid('exec_' . $automation_scenario->id() . '_', TRUE);
    
    $build = [
      '#theme' => 'automation_execution_live',
      '#scenario' => $automation_scenario,
      '#execution_id' => $execution_id,
      '#attached' => [
        'library' => ['ai_web_automation/execution-monitor'],
        'drupalSettings' => [
          'aiWebAutomation' => [
            'executionOptions' => $options,
          ],
        ],
      ],
    ];
    
    return $build;
  }

  /**
   * Gets execution statistics.
   *
   * @return array
   *   Array of statistics.
   */
  protected function getExecutionStatistics() {
    $database = \Drupal::database();
    
    // Total executions.
    $total = $database->select('ai_web_automation_results', 'r')
      ->countQuery()
      ->execute()
      ->fetchField();

    // Successful executions.
    $successful = $database->select('ai_web_automation_results', 'r')
      ->condition('status', 'completed')
      ->countQuery()
      ->execute()
      ->fetchField();

    // Failed executions.
    $failed = $database->select('ai_web_automation_results', 'r')
      ->condition('status', 'failed')
      ->countQuery()
      ->execute()
      ->fetchField();

    // Average duration.
    $avg_duration = $database->select('ai_web_automation_results', 'r')
      ->condition('status', 'completed')
      ->addExpression('AVG(completed - started)', 'avg_duration')
      ->execute()
      ->fetchField();

    return [
      'total' => $total,
      'successful' => $successful,
      'failed' => $failed,
      'success_rate' => $total > 0 ? round(($successful / $total) * 100, 1) : 0,
      'average_duration' => $avg_duration ? round($avg_duration) : 0,
    ];
  }

}