<?php

namespace Drupal\ai_web_automation\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Drupal\ai_web_automation\Service\ScenarioExecutor;
use Drupal\ai_web_automation\Service\ResultsStorage;

/**
 * Controller for handling scenario execution via AJAX.
 */
class ExecutionController extends ControllerBase {

  /**
   * The scenario executor service.
   *
   * @var \Drupal\ai_web_automation\Service\ScenarioExecutor
   */
  protected $scenarioExecutor;

  /**
   * The results storage service.
   *
   * @var \Drupal\ai_web_automation\Service\ResultsStorage
   */
  protected $resultsStorage;

  /**
   * Constructs an ExecutionController object.
   *
   * @param \Drupal\ai_web_automation\Service\ScenarioExecutor $scenario_executor
   *   The scenario executor service.
   * @param \Drupal\ai_web_automation\Service\ResultsStorage $results_storage
   *   The results storage service.
   */
  public function __construct(ScenarioExecutor $scenario_executor, ResultsStorage $results_storage) {
    $this->scenarioExecutor = $scenario_executor;
    $this->resultsStorage = $results_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ai_web_automation.scenario_executor'),
      $container->get('ai_web_automation.results_storage')
    );
  }

  /**
   * Executes a single step via AJAX.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with execution result.
   */
  public function executeStep(AutomationScenarioInterface $automation_scenario, Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    
    if (empty($data['execution_id']) || !isset($data['step_index'])) {
      return new JsonResponse([
        'error' => 'Missing required parameters: execution_id and step_index',
      ], 400);
    }
    
    $execution_id = $data['execution_id'];
    $step_index = (int) $data['step_index'];
    $context = $data['context'] ?? [];
    
    // Get the step configuration.
    $steps = $automation_scenario->getSteps();
    if (!isset($steps[$step_index])) {
      return new JsonResponse([
        'error' => 'Invalid step index',
      ], 400);
    }
    
    // Execute the step.
    $result = $this->scenarioExecutor->executeStep($execution_id, $step_index, $context);
    
    return new JsonResponse([
      'success' => TRUE,
      'result' => $result,
      'next_step' => $step_index + 1 < count($steps) ? $step_index + 1 : NULL,
    ]);
  }

  /**
   * Gets the execution status.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with execution status.
   */
  public function getStatus(Request $request) {
    $execution_id = $request->query->get('execution_id');
    
    if (empty($execution_id)) {
      return new JsonResponse([
        'error' => 'Missing execution_id parameter',
      ], 400);
    }
    
    $status = $this->scenarioExecutor->getExecutionStatus($execution_id);
    
    if (!$status) {
      return new JsonResponse([
        'error' => 'Execution not found',
      ], 404);
    }
    
    return new JsonResponse($status);
  }

  /**
   * Starts a new scenario execution.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with execution ID.
   */
  public function startExecution(AutomationScenarioInterface $automation_scenario, Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    $options = $data['options'] ?? [];
    
    // Start the execution.
    $execution_id = $this->scenarioExecutor->execute($automation_scenario, $options);
    
    if (!$execution_id) {
      return new JsonResponse([
        'error' => 'Failed to start execution',
      ], 500);
    }
    
    return new JsonResponse([
      'success' => TRUE,
      'execution_id' => $execution_id,
      'steps' => $automation_scenario->getSteps(),
    ]);
  }

  /**
   * Cancels an ongoing execution.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response indicating success.
   */
  public function cancelExecution(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    $execution_id = $data['execution_id'] ?? '';
    
    if (empty($execution_id)) {
      return new JsonResponse([
        'error' => 'Missing execution_id',
      ], 400);
    }
    
    // Mark the execution as cancelled.
    $this->resultsStorage->markExecutionFailed($execution_id, 'Execution cancelled by user');
    
    return new JsonResponse([
      'success' => TRUE,
      'message' => 'Execution cancelled',
    ]);
  }

}