<?php

namespace Drupal\ai_web_automation\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Drupal\ai_web_automation\Service\ResultsStorage;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for viewing execution results.
 */
class ResultsController extends ControllerBase {

  /**
   * The results storage service.
   *
   * @var \Drupal\ai_web_automation\Service\ResultsStorage
   */
  protected $resultsStorage;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a ResultsController object.
   *
   * @param \Drupal\ai_web_automation\Service\ResultsStorage $results_storage
   *   The results storage service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ResultsStorage $results_storage, EntityTypeManagerInterface $entity_type_manager) {
    $this->resultsStorage = $results_storage;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ai_web_automation.results_storage'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Displays execution results.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array
   *   Render array for the results page.
   */
  public function view(AutomationScenarioInterface $automation_scenario, $execution_id) {
    $results = $this->resultsStorage->getExecutionResults($execution_id);
    
    if (!$results) {
      throw new NotFoundHttpException('Execution results not found.');
    }
    
    // Load user information.
    $user = NULL;
    if ($results['uid']) {
      $user = $this->entityTypeManager->getStorage('user')->load($results['uid']);
    }
    
    // Load screenshot files.
    $screenshots = [];
    if (!empty($results['screenshots'])) {
      $file_storage = $this->entityTypeManager->getStorage('file');
      $files = $file_storage->loadMultiple($results['screenshots']);
      foreach ($files as $file) {
        $screenshots[] = [
          'file' => $file,
          'url' => $file->createFileUrl(),
        ];
      }
    }
    
    // Format duration.
    $duration = NULL;
    if ($results['completed'] && $results['started']) {
      $duration = $results['completed'] - $results['started'];
    }
    
    $build = [
      '#theme' => 'automation_results',
      '#scenario' => $automation_scenario,
      '#results' => $results,
      '#execution_id' => $execution_id,
      '#user' => $user,
      '#screenshots' => $screenshots,
      '#duration' => $duration,
      '#attached' => [
        'library' => ['ai_web_automation/results'],
      ],
    ];
    
    // Add step details.
    $step_details = [];
    $step_manager = \Drupal::service('ai_web_automation.step_manager');
    
    foreach ($results['step_results'] as $index => $step_result) {
      $step_config = $automation_scenario->getSteps()[$index] ?? NULL;
      if ($step_config) {
        try {
          $step = $step_manager->createStepInstance($step_config['type'], $step_config['config'] ?? []);
          $step_details[$index] = [
            'label' => $step->label(),
            'preview' => $step->preview(),
            'result' => $step_result,
            'config' => $step_config,
          ];
        } catch (\Exception $e) {
          $step_details[$index] = [
            'label' => $this->t('Unknown step'),
            'preview' => $this->t('Step configuration error'),
            'result' => $step_result,
            'config' => $step_config,
          ];
        }
      }
    }
    
    $build['#step_details'] = $step_details;
    
    return $build;
  }

  /**
   * Lists all executions for a scenario.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $automation_scenario
   *   The automation scenario entity.
   *
   * @return array
   *   Render array for the executions list.
   */
  public function listExecutions(AutomationScenarioInterface $automation_scenario) {
    $executions = $this->resultsStorage->getScenarioExecutions($automation_scenario->id(), 50);
    
    $rows = [];
    foreach ($executions as $execution) {
      $user = $this->entityTypeManager->getStorage('user')->load($execution->uid);
      
      $rows[] = [
        'execution_id' => [
          'data' => [
            '#type' => 'link',
            '#title' => substr($execution->execution_id, 0, 8) . '...',
            '#url' => \Drupal\Core\Url::fromRoute('ai_web_automation.results', [
              'automation_scenario' => $automation_scenario->id(),
              'execution_id' => $execution->execution_id,
            ]),
          ],
        ],
        'status' => $execution->status,
        'user' => $user ? $user->toLink() : $this->t('Anonymous'),
        'started' => $this->dateFormatter()->format($execution->started, 'short'),
        'duration' => $execution->completed ? ($execution->completed - $execution->started) . 's' : '-',
      ];
    }
    
    $build = [
      '#type' => 'table',
      '#header' => [
        'execution_id' => $this->t('Execution ID'),
        'status' => $this->t('Status'),
        'user' => $this->t('User'),
        'started' => $this->t('Started'),
        'duration' => $this->t('Duration'),
      ],
      '#rows' => $rows,
      '#empty' => $this->t('No executions found for this scenario.'),
      '#attributes' => ['class' => ['executions-list']],
    ];
    
    return $build;
  }

}