<?php

namespace Drupal\ai_web_automation\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityPublishedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Automation Scenario entity.
 *
 * @ContentEntityType(
 *   id = "automation_scenario",
 *   label = @Translation("Automation Scenario"),
 *   label_collection = @Translation("Automation Scenarios"),
 *   label_singular = @Translation("automation scenario"),
 *   label_plural = @Translation("automation scenarios"),
 *   label_count = @PluralTranslation(
 *     singular = "@count automation scenario",
 *     plural = "@count automation scenarios",
 *   ),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\ai_web_automation\AutomationScenarioListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\ai_web_automation\Form\AutomationScenarioForm",
 *       "add" = "Drupal\ai_web_automation\Form\AutomationScenarioForm",
 *       "edit" = "Drupal\ai_web_automation\Form\AutomationScenarioForm",
 *       "delete" = "Drupal\ai_web_automation\Form\AutomationScenarioDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\ai_web_automation\AutomationScenarioAccessControlHandler",
 *     "permission_provider" = "Drupal\Core\Entity\EntityPermissionProvider",
 *   },
 *   base_table = "automation_scenario",
 *   data_table = "automation_scenario_field_data",
 *   revision_table = "automation_scenario_revision",
 *   revision_data_table = "automation_scenario_field_revision",
 *   translatable = FALSE,
 *   admin_permission = "administer ai web automation",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "uid" = "uid",
 *     "langcode" = "langcode",
 *     "published" = "status",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log"
 *   },
 *   links = {
 *     "canonical" = "/automation-scenario/{automation_scenario}",
 *     "add-form" = "/automation-scenario/add",
 *     "edit-form" = "/automation-scenario/{automation_scenario}/edit",
 *     "delete-form" = "/automation-scenario/{automation_scenario}/delete",
 *     "collection" = "/admin/content/automation-scenarios",
 *     "execute" = "/automation-scenario/{automation_scenario}/execute",
 *     "builder" = "/automation-scenario/{automation_scenario}/builder",
 *   },
 *   field_ui_base_route = "ai_web_automation.settings",
 * )
 */
class AutomationScenario extends ContentEntityBase implements AutomationScenarioInterface {

  use EntityChangedTrait;
  use EntityPublishedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'uid' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->get('description')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setDescription($description) {
    $this->set('description', $description);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSteps() {
    $steps = [];
    foreach ($this->get('steps') as $item) {
      $steps[] = $item->getValue();
    }
    return $steps;
  }

  /**
   * {@inheritdoc}
   */
  public function setSteps(array $steps) {
    $this->set('steps', $steps);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getVariables() {
    $value = $this->get('variables')->first();
    return $value ? $value->getValue() : [];
  }

  /**
   * {@inheritdoc}
   */
  public function setVariables(array $variables) {
    $this->set('variables', $variables);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getLastExecutionTime() {
    $value = $this->get('last_execution')->value;
    return $value ? (int) $value : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setLastExecutionTime($timestamp) {
    $this->set('last_execution', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getExecutionCount() {
    return (int) $this->get('execution_count')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function incrementExecutionCount() {
    $count = $this->getExecutionCount();
    $this->set('execution_count', $count + 1);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getScheduledTime() {
    $value = $this->get('schedule')->value;
    return $value ? strtotime($value) : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setScheduledTime($timestamp) {
    if ($timestamp) {
      $this->set('schedule', date('Y-m-d\TH:i:s', $timestamp));
    }
    else {
      $this->set('schedule', NULL);
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    // Add the published field.
    $fields += static::publishedBaseFieldDefinitions($entity_type);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Automation Scenario entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDefaultValueCallback('Drupal\ai_web_automation\Entity\AutomationScenario::getCurrentUserId')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['status']->setDescription(t('A boolean indicating whether the Automation Scenario is published.'))
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'weight' => -3,
      ]);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    $fields['revision_translation_affected'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Revision translation affected'))
      ->setDescription(t('Indicates if the last edit of a translation belongs to current revision.'))
      ->setReadOnly(TRUE)
      ->setRevisionable(TRUE)
      ->setTranslatable(TRUE);

    return $fields;
  }

  /**
   * Default value callback for 'uid' base field definition.
   *
   * @see ::baseFieldDefinitions()
   *
   * @return array
   *   An array of default values.
   */
  public static function getCurrentUserId() {
    return [\Drupal::currentUser()->id()];
  }

}