<?php

namespace Drupal\ai_web_automation\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for Automation Scenario edit forms.
 */
class AutomationScenarioForm extends ContentEntityForm {

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->stepManager = $container->get('ai_web_automation.step_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    
    // Add library for enhanced UI.
    $form['#attached']['library'][] = 'ai_web_automation/scenario-form';
    
    // Add container for AJAX messages.
    $form['ajax_messages'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'ajax-messages-wrapper'],
      '#weight' => -100,
    ];
    
    // Group basic fields.
    $form['basic'] = [
      '#type' => 'details',
      '#title' => $this->t('Basic information'),
      '#open' => TRUE,
      '#weight' => -10,
    ];
    
    $form['name']['#group'] = 'basic';
    $form['description']['#group'] = 'basic';
    $form['tags']['#group'] = 'basic';
    
    // Group step configuration.
    $form['step_configuration'] = [
      '#type' => 'details',
      '#title' => $this->t('Steps'),
      '#open' => TRUE,
      '#weight' => 0,
      '#attributes' => ['class' => ['step-configuration-wrapper']],
    ];
    
    $form['steps']['#group'] = 'step_configuration';
    
    // Add help text.
    $form['step_configuration']['help'] = [
      '#type' => 'item',
      '#markup' => $this->t('Define the automation steps that will be executed in order. You can use variables like {{variable_name}} in text fields.'),
      '#weight' => -1,
    ];
    
    // Group variables.
    $form['variable_configuration'] = [
      '#type' => 'details',
      '#title' => $this->t('Variables'),
      '#open' => FALSE,
      '#weight' => 10,
    ];
    
    // Custom variables form element since map field doesn't have a default widget.
    $form['variable_configuration']['variables_wrapper'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Variables'),
      '#description' => $this->t('Define variables that can be used in steps with {{variable_name}} syntax.'),
      '#prefix' => '<div id="variables-wrapper">',
      '#suffix' => '</div>',
    ];
    
    $variables = $this->entity->getVariables();
    $num_variables = $form_state->get('num_variables') ?: count($variables) ?: 1;
    $form_state->set('num_variables', $num_variables);
    
    for ($i = 0; $i < $num_variables; $i++) {
      $form['variable_configuration']['variables_wrapper']['variable_' . $i] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['variable-item']],
      ];
      
      $form['variable_configuration']['variables_wrapper']['variable_' . $i]['key'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Variable name'),
        '#default_value' => isset($variables) ? array_keys($variables)[$i] ?? '' : '',
        '#size' => 20,
        '#prefix' => '<div class="container-inline">',
      ];
      
      $form['variable_configuration']['variables_wrapper']['variable_' . $i]['value'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Value'),
        '#default_value' => isset($variables) ? array_values($variables)[$i] ?? '' : '',
        '#size' => 40,
        '#suffix' => '</div>',
      ];
    }
    
    $form['variable_configuration']['variables_wrapper']['add_variable'] = [
      '#type' => 'button',
      '#value' => $this->t('Add variable'),
      '#ajax' => [
        'callback' => '::addVariableCallback',
        'wrapper' => 'variables-wrapper',
      ],
    ];
    
    // Hide the actual variables field.
    $form['variables']['#access'] = FALSE;
    
    // Group advanced settings.
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced settings'),
      '#open' => FALSE,
      '#weight' => 20,
    ];
    
    $form['status']['#group'] = 'advanced';
    $form['schedule']['#group'] = 'advanced';
    $form['uid']['#group'] = 'advanced';
    
    // Add validation button.
    $form['actions']['validate_scenario'] = [
      '#type' => 'button',
      '#value' => $this->t('Validate steps'),
      '#ajax' => [
        'callback' => '::validateSteps',
        'wrapper' => 'ajax-messages-wrapper',
      ],
      '#weight' => -1,
    ];
    
    // Add quick actions.
    $form['actions']['save_and_execute'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save and execute'),
      '#submit' => ['::submitForm', '::saveAndExecute'],
      '#weight' => 1,
    ];
    
    $form['actions']['save_and_builder'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save and open builder'),
      '#submit' => ['::submitForm', '::saveAndOpenBuilder'],
      '#weight' => 2,
    ];
    
    return $form;
  }

  /**
   * AJAX callback to validate steps.
   */
  public function validateSteps(array &$form, FormStateInterface $form_state) {
    $messages = [];
    $steps = $form_state->getValue('steps') ?? [];
    
    foreach ($steps as $delta => $step_data) {
      if (empty($step_data['type'])) {
        continue;
      }
      
      $errors = $this->stepManager->validateStep($step_data['type'], $step_data['config'] ?? []);
      if (!empty($errors)) {
        foreach ($errors as $error) {
          $messages[] = [
            'type' => 'error',
            'message' => $this->t('Step @num (@type): @error', [
              '@num' => $delta + 1,
              '@type' => $step_data['type'],
              '@error' => $error,
            ]),
          ];
        }
      }
    }
    
    if (empty($messages)) {
      $messages[] = [
        'type' => 'status',
        'message' => $this->t('All steps are valid.'),
      ];
    }
    
    $output = ['#type' => 'container'];
    foreach ($messages as $message) {
      $output[] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => [
          'class' => ['messages', 'messages--' . $message['type']],
        ],
        '#value' => $message['message'],
      ];
    }
    
    return $output;
  }

  /**
   * Submit handler for save and execute.
   */
  public function saveAndExecute(array &$form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $form_state->setRedirect('ai_web_automation.execute', ['automation_scenario' => $entity->id()]);
  }

  /**
   * Submit handler for save and open builder.
   */
  public function saveAndOpenBuilder(array &$form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $form_state->setRedirect('ai_web_automation.builder', ['automation_scenario' => $entity->id()]);
  }

  /**
   * AJAX callback to add a variable field.
   */
  public function addVariableCallback(array &$form, FormStateInterface $form_state) {
    $num_variables = $form_state->get('num_variables');
    $form_state->set('num_variables', $num_variables + 1);
    $form_state->setRebuild();
    return $form['variable_configuration']['variables_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Process variables before saving.
    $variables = [];
    $num_variables = $form_state->get('num_variables') ?: 1;
    
    for ($i = 0; $i < $num_variables; $i++) {
      $key = $form_state->getValue(['variable_' . $i, 'key']);
      $value = $form_state->getValue(['variable_' . $i, 'value']);
      
      if (!empty($key)) {
        $variables[$key] = $value;
      }
    }
    
    $this->entity->setVariables($variables);
    
    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;
    $status = parent::save($form, $form_state);
    
    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label automation scenario.', [
          '%label' => $entity->label(),
        ]));
        break;
        
      default:
        $this->messenger()->addMessage($this->t('Saved the %label automation scenario.', [
          '%label' => $entity->label(),
        ]));
    }
    
    $form_state->setRedirect('entity.automation_scenario.canonical', ['automation_scenario' => $entity->id()]);
  }

}