<?php

namespace Drupal\ai_web_automation\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Drupal\ai_web_automation\AutomationStepManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Provides a visual builder form for automation scenarios.
 */
class ScenarioBuilderForm extends FormBase {

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The automation scenario entity.
   *
   * @var \Drupal\ai_web_automation\Entity\AutomationScenarioInterface
   */
  protected $scenario;

  /**
   * Constructs a ScenarioBuilderForm object.
   *
   * @param \Drupal\ai_web_automation\AutomationStepManager $step_manager
   *   The automation step manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(AutomationStepManager $step_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->stepManager = $step_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ai_web_automation.step_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_web_automation_scenario_builder';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, AutomationScenarioInterface $automation_scenario = NULL) {
    $this->scenario = $automation_scenario;
    
    $form['#attached']['library'][] = 'ai_web_automation/scenario-builder';
    $form['#attached']['drupalSettings']['aiWebAutomation'] = [
      'scenarioId' => $automation_scenario->id(),
      'steps' => $automation_scenario->getSteps(),
      'stepTypes' => $this->getStepTypesInfo(),
    ];
    
    $form['builder_container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'scenario-builder-container',
        'class' => ['scenario-builder'],
      ],
    ];
    
    // Header with scenario info
    $form['builder_container']['header'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['builder-header']],
    ];
    
    $form['builder_container']['header']['title'] = [
      '#markup' => '<h2>' . $this->t('Visual Builder: @name', ['@name' => $automation_scenario->getName()]) . '</h2>',
    ];
    
    $form['builder_container']['header']['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['builder-actions']],
    ];
    
    $form['builder_container']['header']['actions']['save'] = [
      '#type' => 'button',
      '#value' => $this->t('Save changes'),
      '#attributes' => ['class' => ['button--primary']],
      '#ajax' => [
        'callback' => '::saveScenario',
        'wrapper' => 'builder-messages',
      ],
    ];
    
    $form['builder_container']['header']['actions']['test'] = [
      '#type' => 'button',
      '#value' => $this->t('Test scenario'),
      '#attributes' => ['class' => ['button']],
    ];
    
    // Messages area
    $form['builder_container']['messages'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'builder-messages',
        'class' => ['builder-messages'],
      ],
    ];
    
    // Main builder area
    $form['builder_container']['builder'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['builder-main']],
    ];
    
    // Step library sidebar
    $form['builder_container']['builder']['sidebar'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['builder-sidebar']],
    ];
    
    $form['builder_container']['builder']['sidebar']['title'] = [
      '#markup' => '<h3>' . $this->t('Step Library') . '</h3>',
    ];
    
    $form['builder_container']['builder']['sidebar']['search'] = [
      '#type' => 'textfield',
      '#placeholder' => $this->t('Search steps...'),
      '#attributes' => [
        'class' => ['step-search'],
        'data-search' => 'steps',
      ],
    ];
    
    $form['builder_container']['builder']['sidebar']['steps'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['step-library']],
    ];
    
    // Group steps by category
    $steps_by_category = $this->stepManager->getStepsByCategory();
    foreach ($steps_by_category as $category => $steps) {
      $form['builder_container']['builder']['sidebar']['steps'][$category] = [
        '#type' => 'details',
        '#title' => ucfirst($category),
        '#open' => TRUE,
        '#attributes' => ['class' => ['step-category']],
      ];
      
      foreach ($steps as $step_id => $step_info) {
        $form['builder_container']['builder']['sidebar']['steps'][$category][$step_id] = [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['step-template'],
            'data-step-type' => $step_id,
            'draggable' => 'true',
          ],
          'content' => [
            '#markup' => '<div class="step-template-content">' .
              '<span class="step-icon step-icon--' . $step_info['icon'] . '"></span>' .
              '<span class="step-label">' . $step_info['label'] . '</span>' .
              '</div>',
          ],
        ];
      }
    }
    
    // Canvas area
    $form['builder_container']['builder']['canvas'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['builder-canvas']],
    ];
    
    $form['builder_container']['builder']['canvas']['steps_container'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'steps-container',
        'class' => ['steps-container'],
      ],
      '#markup' => '<div class="drop-zone-message">' . $this->t('Drag steps here to build your scenario') . '</div>',
    ];
    
    // Properties panel
    $form['builder_container']['builder']['properties'] = [
      '#type' => 'container',
      '#attributes' => [
        'id' => 'properties-panel',
        'class' => ['builder-properties'],
      ],
    ];
    
    $form['builder_container']['builder']['properties']['title'] = [
      '#markup' => '<h3>' . $this->t('Step Properties') . '</h3>',
    ];
    
    $form['builder_container']['builder']['properties']['content'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'properties-content'],
      '#markup' => '<p class="empty-message">' . $this->t('Select a step to edit its properties') . '</p>',
    ];
    
    // Hidden field to store scenario data
    $form['scenario_data'] = [
      '#type' => 'hidden',
      '#attributes' => ['id' => 'scenario-data'],
    ];
    
    // Footer actions
    $form['actions'] = [
      '#type' => 'actions',
    ];
    
    $form['actions']['save_exit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save and exit'),
      '#submit' => ['::saveAndExit'],
    ];
    
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $automation_scenario->toUrl(),
      '#attributes' => ['class' => ['button']],
    ];
    
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // This will be handled by JavaScript and AJAX.
  }

  /**
   * AJAX callback to save the scenario.
   */
  public function saveScenario(array &$form, FormStateInterface $form_state) {
    $scenario_data = json_decode($form_state->getValue('scenario_data'), TRUE);
    
    if ($scenario_data && isset($scenario_data['steps'])) {
      $this->scenario->setSteps($scenario_data['steps']);
      $this->scenario->save();
      
      $message = [
        '#type' => 'status_messages',
        '#weight' => -10,
      ];
    } else {
      $message = [
        '#markup' => '<div class="messages messages--error">' . $this->t('Failed to save scenario data.') . '</div>',
      ];
    }
    
    return $message;
  }

  /**
   * Submit handler for save and exit.
   */
  public function saveAndExit(array &$form, FormStateInterface $form_state) {
    $scenario_data = json_decode($form_state->getValue('scenario_data'), TRUE);
    
    if ($scenario_data && isset($scenario_data['steps'])) {
      $this->scenario->setSteps($scenario_data['steps']);
      $this->scenario->save();
      $this->messenger()->addStatus($this->t('Scenario saved successfully.'));
    }
    
    $form_state->setRedirect('entity.automation_scenario.canonical', [
      'automation_scenario' => $this->scenario->id(),
    ]);
  }

  /**
   * Gets step types information for JavaScript.
   *
   * @return array
   *   Array of step type information.
   */
  protected function getStepTypesInfo() {
    $info = [];
    $definitions = $this->stepManager->getDefinitions();
    
    foreach ($definitions as $id => $definition) {
      $info[$id] = [
        'label' => (string) $definition['label'],
        'description' => (string) $definition['description'],
        'category' => $definition['category'],
        'icon' => $definition['icon'],
      ];
    }
    
    return $info;
  }

}