<?php

namespace Drupal\ai_web_automation\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\ai_web_automation\Service\ScenarioExecutor;

/**
 * Form for executing automation scenarios.
 */
class ScenarioExecuteForm extends FormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The scenario executor service.
   *
   * @var \Drupal\ai_web_automation\Service\ScenarioExecutor
   */
  protected $scenarioExecutor;

  /**
   * The automation scenario entity.
   *
   * @var \Drupal\ai_web_automation\Entity\AutomationScenarioInterface
   */
  protected $scenario;

  /**
   * Constructs a ScenarioExecuteForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\ai_web_automation\Service\ScenarioExecutor $scenario_executor
   *   The scenario executor service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ScenarioExecutor $scenario_executor) {
    $this->entityTypeManager = $entity_type_manager;
    $this->scenarioExecutor = $scenario_executor;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('ai_web_automation.scenario_executor')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_web_automation_scenario_execute';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, AutomationScenarioInterface $automation_scenario = NULL) {
    $this->scenario = $automation_scenario;
    
    $form['#attached']['library'][] = 'ai_web_automation/execution-form';
    
    $form['info'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Scenario Information'),
    ];
    
    $form['info']['name'] = [
      '#type' => 'item',
      '#title' => $this->t('Scenario'),
      '#markup' => $automation_scenario->getName(),
    ];
    
    if ($description = $automation_scenario->getDescription()) {
      $form['info']['description'] = [
        '#type' => 'item',
        '#title' => $this->t('Description'),
        '#markup' => $description,
      ];
    }
    
    $steps = $automation_scenario->getSteps();
    $form['info']['step_count'] = [
      '#type' => 'item',
      '#title' => $this->t('Number of steps'),
      '#markup' => count($steps),
    ];
    
    // Variable overrides.
    $variables = $automation_scenario->getVariables();
    if (!empty($variables)) {
      $form['variables'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Variables'),
        '#description' => $this->t('Override scenario variables for this execution.'),
        '#tree' => TRUE,
      ];
      
      foreach ($variables as $key => $value) {
        $form['variables'][$key] = [
          '#type' => 'textfield',
          '#title' => $key,
          '#default_value' => $value,
          '#description' => $this->t('Use as {{@key}} in steps', ['@key' => $key]),
        ];
      }
    }
    
    // Execution options.
    $form['options'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Execution Options'),
    ];
    
    $form['options']['mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Execution mode'),
      '#options' => [
        'normal' => $this->t('Normal - Execute all steps'),
        'dry_run' => $this->t('Dry run - Preview without execution'),
        'step_by_step' => $this->t('Step by step - Pause between steps'),
      ],
      '#default_value' => 'normal',
    ];
    
    $form['options']['capture_screenshots'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Capture screenshots after each step'),
      '#default_value' => FALSE,
    ];
    
    $form['options']['stop_on_error'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Stop execution on first error'),
      '#default_value' => TRUE,
    ];
    
    $config = $this->config('ai_web_automation.settings');
    $browser_context = $config->get('browser_context');
    
    $form['options']['headless'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Run in headless mode'),
      '#default_value' => $browser_context['headless'] ?? TRUE,
      '#description' => $this->t('Uncheck to see the browser window during execution.'),
    ];
    
    // Actions.
    $form['actions'] = [
      '#type' => 'actions',
    ];
    
    $form['actions']['execute'] = [
      '#type' => 'submit',
      '#value' => $this->t('Execute'),
      '#attributes' => ['class' => ['button--primary']],
    ];
    
    $form['actions']['execute_live'] = [
      '#type' => 'submit',
      '#value' => $this->t('Execute with live view'),
      '#submit' => ['::executeLive'],
    ];
    
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $automation_scenario->toUrl(),
      '#attributes' => ['class' => ['button']],
    ];
    
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $options = [
      'mode' => $form_state->getValue(['options', 'mode']),
      'capture_screenshots' => $form_state->getValue(['options', 'capture_screenshots']),
      'stop_on_error' => $form_state->getValue(['options', 'stop_on_error']),
      'headless' => $form_state->getValue(['options', 'headless']),
      'variables' => $form_state->getValue('variables') ?? [],
    ];
    
    // Execute the scenario.
    $execution_id = $this->scenarioExecutor->execute($this->scenario, $options);
    
    if ($execution_id) {
      $this->messenger()->addStatus($this->t('Scenario execution started. Execution ID: @id', ['@id' => $execution_id]));
      $form_state->setRedirect('ai_web_automation.results', [
        'automation_scenario' => $this->scenario->id(),
        'execution_id' => $execution_id,
      ]);
    } else {
      $this->messenger()->addError($this->t('Failed to start scenario execution.'));
    }
  }

  /**
   * Submit handler for live execution.
   */
  public function executeLive(array &$form, FormStateInterface $form_state) {
    $options = [
      'mode' => $form_state->getValue(['options', 'mode']),
      'capture_screenshots' => $form_state->getValue(['options', 'capture_screenshots']),
      'stop_on_error' => $form_state->getValue(['options', 'stop_on_error']),
      'headless' => $form_state->getValue(['options', 'headless']),
      'variables' => $form_state->getValue('variables') ?? [],
      'live_view' => TRUE,
    ];
    
    // Store options in session for live view.
    $session = $this->getRequest()->getSession();
    $session->set('ai_web_automation_execution_options', $options);
    
    // Redirect to live execution page.
    $form_state->setRedirect('ai_web_automation.execute_live', [
      'automation_scenario' => $this->scenario->id(),
    ]);
  }

}