<?php

namespace Drupal\ai_web_automation\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\Service\PlaywrightMcpService;

/**
 * Configure AI Web Automation settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The MCP client service.
   *
   * @var \Drupal\ai_web_automation\Service\PlaywrightMcpService
   */
  protected $mcpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->mcpClient = $container->get('ai_web_automation.mcp_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ai_web_automation_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['ai_web_automation.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('ai_web_automation.settings');

    $form['mcp'] = [
      '#type' => 'details',
      '#title' => $this->t('MCP Server Configuration'),
      '#open' => TRUE,
    ];

    $form['mcp']['mcp_server_url'] = [
      '#type' => 'url',
      '#title' => $this->t('MCP Server URL'),
      '#description' => $this->t('The URL of the Playwright MCP server.'),
      '#default_value' => $config->get('mcp_server_url'),
      '#required' => TRUE,
    ];

    $form['mcp']['mcp_api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('API key for authenticating with the MCP server.'),
      '#default_value' => $config->get('mcp_api_key'),
    ];

    $form['mcp']['test_connection'] = [
      '#type' => 'button',
      '#value' => $this->t('Test Connection'),
      '#ajax' => [
        'callback' => '::testConnection',
        'wrapper' => 'connection-test-result',
      ],
    ];

    $form['mcp']['test_result'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'connection-test-result'],
    ];

    $form['browser'] = [
      '#type' => 'details',
      '#title' => $this->t('Browser Settings'),
      '#open' => TRUE,
    ];

    $browser_context = $config->get('browser_context');

    $form['browser']['new_instance'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Create new browser instance for each execution'),
      '#description' => $this->t('If unchecked, browser sessions will be reused when possible.'),
      '#default_value' => $browser_context['new_instance'] ?? TRUE,
    ];

    $form['browser']['headless'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Run browser in headless mode by default'),
      '#description' => $this->t('Headless mode runs without a visible browser window.'),
      '#default_value' => $browser_context['headless'] ?? TRUE,
    ];

    $form['browser']['viewport'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Default Viewport Size'),
    ];

    $form['browser']['viewport']['width'] = [
      '#type' => 'number',
      '#title' => $this->t('Width'),
      '#default_value' => $browser_context['viewport']['width'] ?? 1280,
      '#min' => 320,
      '#max' => 3840,
      '#step' => 10,
    ];

    $form['browser']['viewport']['height'] = [
      '#type' => 'number',
      '#title' => $this->t('Height'),
      '#default_value' => $browser_context['viewport']['height'] ?? 720,
      '#min' => 240,
      '#max' => 2160,
      '#step' => 10,
    ];

    $form['storage'] = [
      '#type' => 'details',
      '#title' => $this->t('Storage Settings'),
      '#open' => FALSE,
    ];

    $storage = $config->get('storage');

    $form['storage']['screenshots'] = [
      '#type' => 'select',
      '#title' => $this->t('Screenshot storage'),
      '#options' => [
        'temporary' => $this->t('Temporary (deleted after 30 days)'),
        'permanent' => $this->t('Permanent'),
        'none' => $this->t('Do not store screenshots'),
      ],
      '#default_value' => $storage['screenshots'] ?? 'temporary',
    ];

    $form['storage']['results_retention'] = [
      '#type' => 'number',
      '#title' => $this->t('Results retention (days)'),
      '#description' => $this->t('Number of days to keep execution results. Set to 0 to keep indefinitely.'),
      '#default_value' => $storage['results_retention'] ?? 30,
      '#min' => 0,
      '#max' => 365,
    ];

    $form['security'] = [
      '#type' => 'details',
      '#title' => $this->t('Security Settings'),
      '#open' => FALSE,
    ];

    $security = $config->get('security');

    $form['security']['allow_javascript'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow custom JavaScript execution'),
      '#description' => $this->t('Warning: Enabling this allows scenarios to execute arbitrary JavaScript.'),
      '#default_value' => $security['allow_javascript'] ?? FALSE,
    ];

    $form['security']['url_whitelist'] = [
      '#type' => 'textarea',
      '#title' => $this->t('URL Whitelist'),
      '#description' => $this->t('One pattern per line. Use * as wildcard. Leave empty to allow all URLs.'),
      '#default_value' => implode("\n", $security['url_whitelist'] ?? []),
    ];

    $form['security']['url_blacklist'] = [
      '#type' => 'textarea',
      '#title' => $this->t('URL Blacklist'),
      '#description' => $this->t('One pattern per line. Use * as wildcard. These URLs will be blocked even if whitelisted.'),
      '#default_value' => implode("\n", $security['url_blacklist'] ?? []),
    ];

    $form['execution'] = [
      '#type' => 'details',
      '#title' => $this->t('Execution Settings'),
      '#open' => FALSE,
    ];

    $execution = $config->get('execution');

    $form['execution']['max_concurrent'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum concurrent executions'),
      '#description' => $this->t('Maximum number of scenarios that can run simultaneously.'),
      '#default_value' => $execution['max_concurrent'] ?? 3,
      '#min' => 1,
      '#max' => 10,
    ];

    $form['execution']['queue_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable queue for background execution'),
      '#description' => $this->t('Use Drupal queue for long-running scenarios.'),
      '#default_value' => $execution['queue_enabled'] ?? TRUE,
    ];

    $form['execution']['allow_scheduling'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow scheduled execution'),
      '#description' => $this->t('Enable scheduling scenarios to run at specific times.'),
      '#default_value' => $execution['allow_scheduling'] ?? TRUE,
    ];

    $form['execution']['default_timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Default timeout (ms)'),
      '#description' => $this->t('Default timeout for MCP operations in milliseconds.'),
      '#default_value' => $config->get('default_timeout') ?? 30000,
      '#min' => 5000,
      '#max' => 300000,
      '#step' => 1000,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to test MCP connection.
   */
  public function testConnection(array &$form, FormStateInterface $form_state) {
    // Save temporary config for testing.
    $temp_config = [
      'mcp_server_url' => $form_state->getValue('mcp_server_url'),
      'mcp_api_key' => $form_state->getValue('mcp_api_key'),
    ];
    
    // Test connection.
    $result = $this->mcpClient->testConnection();
    
    $output = [
      '#type' => 'container',
      '#attributes' => ['id' => 'connection-test-result'],
    ];
    
    if ($result['success']) {
      $output['message'] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => ['class' => ['messages', 'messages--status']],
        '#value' => $result['message'],
      ];
    } else {
      $output['message'] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#attributes' => ['class' => ['messages', 'messages--error']],
        '#value' => $result['message'],
      ];
    }
    
    return $output;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('ai_web_automation.settings');
    
    // MCP settings.
    $config->set('mcp_server_url', $form_state->getValue('mcp_server_url'));
    $config->set('mcp_api_key', $form_state->getValue('mcp_api_key'));
    $config->set('default_timeout', $form_state->getValue('default_timeout'));
    
    // Browser settings.
    $config->set('browser_context.new_instance', $form_state->getValue('new_instance'));
    $config->set('browser_context.headless', $form_state->getValue('headless'));
    $config->set('browser_context.viewport.width', $form_state->getValue(['viewport', 'width']));
    $config->set('browser_context.viewport.height', $form_state->getValue(['viewport', 'height']));
    
    // Storage settings.
    $config->set('storage.screenshots', $form_state->getValue('screenshots'));
    $config->set('storage.results_retention', $form_state->getValue('results_retention'));
    
    // Security settings.
    $config->set('security.allow_javascript', $form_state->getValue('allow_javascript'));
    
    // Process URL lists.
    $whitelist = array_filter(array_map('trim', explode("\n", $form_state->getValue('url_whitelist'))));
    $blacklist = array_filter(array_map('trim', explode("\n", $form_state->getValue('url_blacklist'))));
    $config->set('security.url_whitelist', $whitelist);
    $config->set('security.url_blacklist', $blacklist);
    
    // Execution settings.
    $config->set('execution.max_concurrent', $form_state->getValue('max_concurrent'));
    $config->set('execution.queue_enabled', $form_state->getValue('queue_enabled'));
    $config->set('execution.allow_scheduling', $form_state->getValue('allow_scheduling'));
    
    $config->save();
    
    parent::submitForm($form, $form_state);
  }

}