<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a click step.
 *
 * @AutomationStep(
 *   id = "click",
 *   label = @Translation("Click"),
 *   description = @Translation("Click on an element"),
 *   category = "interaction",
 *   icon = "click",
 *   weight = 0,
 *   mcp_tool = "playwright_click"
 * )
 */
class ClickStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'selector' => '',
      'button' => 'left',
      'click_count' => 1,
      'wait_for_navigation' => FALSE,
      'force' => FALSE,
      'timeout' => 30000,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Selector'),
      '#description' => $this->t('CSS selector, XPath, or text to click. Supports variables.'),
      '#default_value' => $this->configuration['selector'],
      '#required' => TRUE,
      '#size' => 60,
    ];

    $form['button'] = [
      '#type' => 'select',
      '#title' => $this->t('Mouse button'),
      '#options' => [
        'left' => $this->t('Left'),
        'right' => $this->t('Right'),
        'middle' => $this->t('Middle'),
      ],
      '#default_value' => $this->configuration['button'],
    ];

    $form['click_count'] = [
      '#type' => 'number',
      '#title' => $this->t('Click count'),
      '#description' => $this->t('Number of times to click (e.g., 2 for double-click).'),
      '#default_value' => $this->configuration['click_count'],
      '#min' => 1,
      '#max' => 3,
    ];

    $form['wait_for_navigation'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Wait for navigation'),
      '#description' => $this->t('Wait for navigation after clicking.'),
      '#default_value' => $this->configuration['wait_for_navigation'],
    ];

    $form['force'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Force click'),
      '#description' => $this->t('Force click even if element is not visible or clickable.'),
      '#default_value' => $this->configuration['force'],
    ];

    $form['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout (ms)'),
      '#description' => $this->t('Maximum time to wait for element.'),
      '#default_value' => $this->configuration['timeout'],
      '#min' => 0,
      '#max' => 300000,
      '#step' => 1000,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['selector'] = $form_state->getValue('selector');
    $this->configuration['button'] = $form_state->getValue('button');
    $this->configuration['click_count'] = $form_state->getValue('click_count');
    $this->configuration['wait_for_navigation'] = $form_state->getValue('wait_for_navigation');
    $this->configuration['force'] = $form_state->getValue('force');
    $this->configuration['timeout'] = $form_state->getValue('timeout');
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    
    if (empty($this->configuration['selector'])) {
      $errors[] = $this->t('Selector is required.');
    }
    
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $selector) {
      return $mcp_client->execute([
        'tool' => $this->getMcpTool(),
        'parameters' => [
          'selector' => $selector,
          'button' => $this->configuration['button'],
          'clickCount' => $this->configuration['click_count'],
          'force' => $this->configuration['force'],
          'timeout' => $this->configuration['timeout'],
          'waitForNavigation' => $this->configuration['wait_for_navigation'],
        ],
      ]);
    });
    
    $result = $measured['result'];
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'selector' => $selector,
        'element_found' => !empty($result['data']['element_found']),
      ],
      'error' => $result['error'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    $selector = $this->configuration['selector'];
    if ($this->configuration['click_count'] > 1) {
      return $this->t('@button-click @count times on @selector', [
        '@button' => ucfirst($this->configuration['button']),
        '@count' => $this->configuration['click_count'],
        '@selector' => $selector,
      ]);
    }
    return $this->t('@button-click on @selector', [
      '@button' => ucfirst($this->configuration['button']),
      '@selector' => $selector,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Click submit button'),
        'config' => [
          'selector' => '#submit-button',
          'wait_for_navigation' => TRUE,
        ],
      ],
      [
        'label' => $this->t('Click by text'),
        'config' => [
          'selector' => 'text=Sign In',
        ],
      ],
      [
        'label' => $this->t('Double-click element'),
        'config' => [
          'selector' => '.item',
          'click_count' => 2,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return 'playwright_click';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    return [
      'selector' => $this->interpolateVariables($this->configuration['selector'], $variables),
      'button' => $this->configuration['button'],
      'clickCount' => $this->configuration['click_count'],
      'force' => $this->configuration['force'],
      'timeout' => $this->configuration['timeout'],
      'waitForNavigation' => $this->configuration['wait_for_navigation'],
    ];
  }

}