<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a data extraction step.
 *
 * @AutomationStep(
 *   id = "extract",
 *   label = @Translation("Extract"),
 *   description = @Translation("Extract data from the page"),
 *   category = "data",
 *   icon = "extract",
 *   weight = 0,
 *   mcp_tool = "playwright_evaluate"
 * )
 */
class ExtractStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'extraction_type' => 'text',
      'selector' => '',
      'attribute' => '',
      'variable_name' => '',
      'multiple' => FALSE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['variable_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Variable name'),
      '#description' => $this->t('Name of the variable to store the extracted data.'),
      '#default_value' => $this->configuration['variable_name'],
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['extraction_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Extraction type'),
      '#options' => [
        'text' => $this->t('Text content'),
        'attribute' => $this->t('Attribute value'),
        'html' => $this->t('Inner HTML'),
        'value' => $this->t('Input value'),
        'url' => $this->t('Current URL'),
        'title' => $this->t('Page title'),
      ],
      '#default_value' => $this->configuration['extraction_type'],
      '#required' => TRUE,
    ];

    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element selector'),
      '#description' => $this->t('CSS selector or XPath of the element.'),
      '#default_value' => $this->configuration['selector'],
      '#size' => 60,
      '#states' => [
        'visible' => [
          ':input[name="extraction_type"]' => [
            ['value' => 'text'],
            ['value' => 'attribute'],
            ['value' => 'html'],
            ['value' => 'value'],
          ],
        ],
        'required' => [
          ':input[name="extraction_type"]' => [
            ['value' => 'text'],
            ['value' => 'attribute'],
            ['value' => 'html'],
            ['value' => 'value'],
          ],
        ],
      ],
    ];

    $form['attribute'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Attribute name'),
      '#description' => $this->t('Name of the attribute to extract (e.g., href, src, data-id).'),
      '#default_value' => $this->configuration['attribute'],
      '#states' => [
        'visible' => [
          ':input[name="extraction_type"]' => ['value' => 'attribute'],
        ],
        'required' => [
          ':input[name="extraction_type"]' => ['value' => 'attribute'],
        ],
      ],
    ];

    $form['multiple'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Extract from multiple elements'),
      '#description' => $this->t('Extract data from all matching elements as an array.'),
      '#default_value' => $this->configuration['multiple'],
      '#states' => [
        'visible' => [
          ':input[name="extraction_type"]' => [
            ['value' => 'text'],
            ['value' => 'attribute'],
            ['value' => 'html'],
            ['value' => 'value'],
          ],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['variable_name'] = $form_state->getValue('variable_name');
    $this->configuration['extraction_type'] = $form_state->getValue('extraction_type');
    $this->configuration['selector'] = $form_state->getValue('selector');
    $this->configuration['attribute'] = $form_state->getValue('attribute');
    $this->configuration['multiple'] = $form_state->getValue('multiple');
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    
    if (empty($this->configuration['variable_name'])) {
      $errors[] = $this->t('Variable name is required.');
    }
    
    $needs_selector = ['text', 'attribute', 'html', 'value'];
    if (in_array($this->configuration['extraction_type'], $needs_selector) && empty($this->configuration['selector'])) {
      $errors[] = $this->t('Selector is required for @type extraction.', ['@type' => $this->configuration['extraction_type']]);
    }
    
    if ($this->configuration['extraction_type'] === 'attribute' && empty($this->configuration['attribute'])) {
      $errors[] = $this->t('Attribute name is required.');
    }
    
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    
    // Build JavaScript based on extraction type.
    $js = $this->buildExtractionScript($selector);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $js) {
      return $mcp_client->execute([
        'tool' => 'playwright_evaluate',
        'parameters' => [
          'expression' => $js,
        ],
      ]);
    });
    
    $result = $measured['result'];
    
    if (!empty($result['success'])) {
      // Store the extracted value in context for future steps.
      $context['variables'][$this->configuration['variable_name']] = $result['data']['value'];
    }
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'variable_name' => $this->configuration['variable_name'],
        'value' => $result['data']['value'] ?? NULL,
        'type' => gettype($result['data']['value'] ?? NULL),
      ],
      'error' => $result['error'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * Builds the JavaScript extraction script.
   *
   * @param string $selector
   *   The element selector.
   *
   * @return string
   *   The JavaScript code.
   */
  protected function buildExtractionScript($selector) {
    switch ($this->configuration['extraction_type']) {
      case 'url':
        return 'window.location.href';
        
      case 'title':
        return 'document.title';
        
      case 'text':
        if ($this->configuration['multiple']) {
          return "Array.from(document.querySelectorAll('$selector')).map(el => el.textContent.trim())";
        }
        return "document.querySelector('$selector')?.textContent?.trim() || ''";
        
      case 'attribute':
        $attr = $this->configuration['attribute'];
        if ($this->configuration['multiple']) {
          return "Array.from(document.querySelectorAll('$selector')).map(el => el.getAttribute('$attr'))";
        }
        return "document.querySelector('$selector')?.getAttribute('$attr') || ''";
        
      case 'html':
        if ($this->configuration['multiple']) {
          return "Array.from(document.querySelectorAll('$selector')).map(el => el.innerHTML)";
        }
        return "document.querySelector('$selector')?.innerHTML || ''";
        
      case 'value':
        if ($this->configuration['multiple']) {
          return "Array.from(document.querySelectorAll('$selector')).map(el => el.value)";
        }
        return "document.querySelector('$selector')?.value || ''";
        
      default:
        return "''";
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    $type_labels = [
      'text' => $this->t('text'),
      'attribute' => $this->t('@attr attribute', ['@attr' => $this->configuration['attribute']]),
      'html' => $this->t('HTML'),
      'value' => $this->t('value'),
      'url' => $this->t('URL'),
      'title' => $this->t('page title'),
    ];
    
    $type = $type_labels[$this->configuration['extraction_type']] ?? $this->configuration['extraction_type'];
    
    if (in_array($this->configuration['extraction_type'], ['url', 'title'])) {
      return $this->t('Extract @type into {{@var}}', [
        '@type' => $type,
        '@var' => $this->configuration['variable_name'],
      ]);
    }
    
    return $this->t('Extract @type from @selector into {{@var}}', [
      '@type' => $type,
      '@selector' => $this->configuration['selector'],
      '@var' => $this->configuration['variable_name'],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function supportsConditions() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Extract page title'),
        'config' => [
          'extraction_type' => 'title',
          'variable_name' => 'page_title',
        ],
      ],
      [
        'label' => $this->t('Extract link URLs'),
        'config' => [
          'extraction_type' => 'attribute',
          'selector' => 'a.product-link',
          'attribute' => 'href',
          'variable_name' => 'product_links',
          'multiple' => TRUE,
        ],
      ],
      [
        'label' => $this->t('Extract form value'),
        'config' => [
          'extraction_type' => 'value',
          'selector' => '#username',
          'variable_name' => 'current_username',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return 'playwright_evaluate';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    return [
      'expression' => $this->buildExtractionScript($selector),
    ];
  }

}