<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a navigation step.
 *
 * @AutomationStep(
 *   id = "navigate",
 *   label = @Translation("Navigate"),
 *   description = @Translation("Navigate to a URL"),
 *   category = "navigation",
 *   icon = "navigate",
 *   weight = -10,
 *   mcp_tool = "playwright_navigate"
 * )
 */
class NavigateStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'url' => '',
      'wait_until' => 'load',
      'timeout' => 30000,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#description' => $this->t('The URL to navigate to. Supports variables like {{base_url}}.'),
      '#default_value' => $this->configuration['url'],
      '#required' => TRUE,
      '#size' => 60,
      '#maxlength' => 2048,
    ];

    $form['wait_until'] = [
      '#type' => 'select',
      '#title' => $this->t('Wait until'),
      '#description' => $this->t('When to consider navigation succeeded.'),
      '#options' => [
        'load' => $this->t('Load event fired'),
        'domcontentloaded' => $this->t('DOM content loaded'),
        'networkidle' => $this->t('Network idle (no requests for 500ms)'),
        'commit' => $this->t('Navigation committed'),
      ],
      '#default_value' => $this->configuration['wait_until'],
    ];

    $form['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout (ms)'),
      '#description' => $this->t('Maximum time to wait for navigation.'),
      '#default_value' => $this->configuration['timeout'],
      '#min' => 0,
      '#max' => 300000,
      '#step' => 1000,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $url = $form_state->getValue('url');
    
    // Allow variables in URL.
    $test_url = preg_replace('/\{\{[^}]+\}\}/', 'https://example.com', $url);
    
    if (!filter_var($test_url, FILTER_VALIDATE_URL) && !str_starts_with($test_url, '/')) {
      $form_state->setErrorByName('url', $this->t('Please enter a valid URL or path.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['url'] = $form_state->getValue('url');
    $this->configuration['wait_until'] = $form_state->getValue('wait_until');
    $this->configuration['timeout'] = $form_state->getValue('timeout');
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    
    if (empty($this->configuration['url'])) {
      $errors[] = $this->t('URL is required.');
    }
    
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    $url = $this->interpolateVariables($this->configuration['url'], $variables);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $url) {
      return $mcp_client->execute([
        'tool' => $this->getMcpTool(),
        'parameters' => [
          'url' => $url,
          'waitUntil' => $this->configuration['wait_until'],
          'timeout' => $this->configuration['timeout'],
        ],
      ]);
    });
    
    $result = $measured['result'];
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'url' => $url,
        'final_url' => $result['data']['url'] ?? $url,
        'title' => $result['data']['title'] ?? '',
      ],
      'error' => $result['error'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    return $this->t('Navigate to @url', ['@url' => $this->configuration['url']]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Navigate to homepage'),
        'config' => [
          'url' => 'https://example.com',
          'wait_until' => 'networkidle',
        ],
      ],
      [
        'label' => $this->t('Navigate with variable'),
        'config' => [
          'url' => '{{base_url}}/login',
          'wait_until' => 'load',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return 'playwright_navigate';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    return [
      'url' => $this->interpolateVariables($this->configuration['url'], $variables),
      'waitUntil' => $this->configuration['wait_until'],
      'timeout' => $this->configuration['timeout'],
    ];
  }

}