<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a screenshot step.
 *
 * @AutomationStep(
 *   id = "screenshot",
 *   label = @Translation("Screenshot"),
 *   description = @Translation("Take a screenshot of the page"),
 *   category = "utility",
 *   icon = "screenshot",
 *   weight = 10,
 *   mcp_tool = "playwright_screenshot"
 * )
 */
class ScreenshotStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'name' => '',
      'full_page' => TRUE,
      'selector' => '',
      'quality' => 80,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Screenshot name'),
      '#description' => $this->t('Name for the screenshot file. Leave empty for auto-generated name.'),
      '#default_value' => $this->configuration['name'],
    ];

    $form['full_page'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Full page screenshot'),
      '#description' => $this->t('Capture the entire page instead of just the viewport.'),
      '#default_value' => $this->configuration['full_page'],
      '#states' => [
        'disabled' => [
          ':input[name="selector"]' => ['filled' => TRUE],
        ],
      ],
    ];

    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element selector'),
      '#description' => $this->t('Optional: Capture only a specific element.'),
      '#default_value' => $this->configuration['selector'],
      '#states' => [
        'disabled' => [
          ':input[name="full_page"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['quality'] = [
      '#type' => 'number',
      '#title' => $this->t('Quality'),
      '#description' => $this->t('JPEG quality (0-100). Only applies to JPEG format.'),
      '#default_value' => $this->configuration['quality'],
      '#min' => 0,
      '#max' => 100,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['name'] = $form_state->getValue('name');
    $this->configuration['full_page'] = $form_state->getValue('full_page');
    $this->configuration['selector'] = $form_state->getValue('selector');
    $this->configuration['quality'] = $form_state->getValue('quality');
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    $name = $this->interpolateVariables($this->configuration['name'], $variables);
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $name, $selector) {
      $parameters = [
        'type' => 'jpeg',
        'quality' => $this->configuration['quality'],
      ];
      
      if (!empty($name)) {
        $parameters['path'] = $name . '.jpg';
      }
      
      if (!empty($selector)) {
        $parameters['selector'] = $selector;
      } else {
        $parameters['fullPage'] = $this->configuration['full_page'];
      }
      
      return $mcp_client->execute([
        'tool' => $this->getMcpTool(),
        'parameters' => $parameters,
      ]);
    });
    
    $result = $measured['result'];
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'screenshot_path' => $result['data']['path'] ?? NULL,
        'screenshot_data' => $result['data']['data'] ?? NULL,
      ],
      'error' => $result['error'] ?? NULL,
      'screenshot' => $result['data']['file_id'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    if (!empty($this->configuration['selector'])) {
      return $this->t('Take screenshot of @selector', ['@selector' => $this->configuration['selector']]);
    }
    return $this->configuration['full_page'] 
      ? $this->t('Take full page screenshot')
      : $this->t('Take viewport screenshot');
  }

  /**
   * {@inheritdoc}
   */
  public function capturesScreenshot() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Full page screenshot'),
        'config' => [
          'full_page' => TRUE,
          'name' => 'full-page',
        ],
      ],
      [
        'label' => $this->t('Element screenshot'),
        'config' => [
          'selector' => '#main-content',
          'name' => 'main-content',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return 'playwright_screenshot';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    $parameters = [
      'type' => 'jpeg',
      'quality' => $this->configuration['quality'],
    ];
    
    if (!empty($this->configuration['name'])) {
      $parameters['path'] = $this->interpolateVariables($this->configuration['name'], $variables) . '.jpg';
    }
    
    if (!empty($this->configuration['selector'])) {
      $parameters['selector'] = $this->interpolateVariables($this->configuration['selector'], $variables);
    } else {
      $parameters['fullPage'] = $this->configuration['full_page'];
    }
    
    return $parameters;
  }

}