<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a type/fill step.
 *
 * @AutomationStep(
 *   id = "type",
 *   label = @Translation("Type"),
 *   description = @Translation("Type text into an input field"),
 *   category = "interaction",
 *   icon = "type",
 *   weight = 1,
 *   mcp_tool = "playwright_fill"
 * )
 */
class TypeStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'selector' => '',
      'text' => '',
      'clear_first' => TRUE,
      'press_enter' => FALSE,
      'delay' => 0,
      'timeout' => 30000,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Selector'),
      '#description' => $this->t('CSS selector or XPath of the input field.'),
      '#default_value' => $this->configuration['selector'],
      '#required' => TRUE,
      '#size' => 60,
    ];

    $form['text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Text'),
      '#description' => $this->t('Text to type. Supports variables like {{username}}.'),
      '#default_value' => $this->configuration['text'],
      '#required' => TRUE,
      '#size' => 60,
    ];

    $form['clear_first'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Clear field first'),
      '#description' => $this->t('Clear existing content before typing.'),
      '#default_value' => $this->configuration['clear_first'],
    ];

    $form['press_enter'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Press Enter after typing'),
      '#description' => $this->t('Simulate pressing Enter key after typing.'),
      '#default_value' => $this->configuration['press_enter'],
    ];

    $form['delay'] = [
      '#type' => 'number',
      '#title' => $this->t('Typing delay (ms)'),
      '#description' => $this->t('Delay between keystrokes to simulate human typing.'),
      '#default_value' => $this->configuration['delay'],
      '#min' => 0,
      '#max' => 1000,
      '#step' => 10,
    ];

    $form['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout (ms)'),
      '#description' => $this->t('Maximum time to wait for element.'),
      '#default_value' => $this->configuration['timeout'],
      '#min' => 0,
      '#max' => 300000,
      '#step' => 1000,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['selector'] = $form_state->getValue('selector');
    $this->configuration['text'] = $form_state->getValue('text');
    $this->configuration['clear_first'] = $form_state->getValue('clear_first');
    $this->configuration['press_enter'] = $form_state->getValue('press_enter');
    $this->configuration['delay'] = $form_state->getValue('delay');
    $this->configuration['timeout'] = $form_state->getValue('timeout');
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    
    if (empty($this->configuration['selector'])) {
      $errors[] = $this->t('Selector is required.');
    }
    
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    $text = $this->interpolateVariables($this->configuration['text'], $variables);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $selector, $text) {
      $parameters = [
        'selector' => $selector,
        'value' => $text,
        'timeout' => $this->configuration['timeout'],
      ];
      
      // Handle clear and type with delay.
      if ($this->configuration['clear_first']) {
        // Clear the field first.
        $mcp_client->execute([
          'tool' => 'playwright_fill',
          'parameters' => array_merge($parameters, ['value' => '']),
        ]);
      }
      
      if ($this->configuration['delay'] > 0) {
        // Use type method for delayed typing.
        $result = $mcp_client->execute([
          'tool' => 'playwright_type',
          'parameters' => [
            'selector' => $selector,
            'text' => $text,
            'delay' => $this->configuration['delay'],
            'timeout' => $this->configuration['timeout'],
          ],
        ]);
      } else {
        // Use fill for instant typing.
        $result = $mcp_client->execute([
          'tool' => 'playwright_fill',
          'parameters' => $parameters,
        ]);
      }
      
      // Press Enter if requested.
      if ($this->configuration['press_enter']) {
        $mcp_client->execute([
          'tool' => 'playwright_press',
          'parameters' => [
            'selector' => $selector,
            'key' => 'Enter',
          ],
        ]);
      }
      
      return $result;
    });
    
    $result = $measured['result'];
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'selector' => $selector,
        'text_length' => mb_strlen($text),
      ],
      'error' => $result['error'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    $text = $this->configuration['text'];
    // Truncate long text for preview.
    if (mb_strlen($text) > 30) {
      $text = mb_substr($text, 0, 27) . '...';
    }
    return $this->t('Type "@text" into @selector', [
      '@text' => $text,
      '@selector' => $this->configuration['selector'],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Fill username field'),
        'config' => [
          'selector' => '#username',
          'text' => '{{username}}',
          'clear_first' => TRUE,
        ],
      ],
      [
        'label' => $this->t('Search with Enter'),
        'config' => [
          'selector' => 'input[type="search"]',
          'text' => 'Drupal automation',
          'press_enter' => TRUE,
        ],
      ],
      [
        'label' => $this->t('Type with delay'),
        'config' => [
          'selector' => '#message',
          'text' => 'Hello, this is typed slowly',
          'delay' => 100,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return $this->configuration['delay'] > 0 ? 'playwright_type' : 'playwright_fill';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    $text = $this->interpolateVariables($this->configuration['text'], $variables);
    
    if ($this->configuration['delay'] > 0) {
      return [
        'selector' => $selector,
        'text' => $text,
        'delay' => $this->configuration['delay'],
        'timeout' => $this->configuration['timeout'],
      ];
    }
    
    return [
      'selector' => $selector,
      'value' => $text,
      'timeout' => $this->configuration['timeout'],
    ];
  }

}