<?php

namespace Drupal\ai_web_automation\Plugin\AutomationStep;

use Drupal\ai_web_automation\AutomationStepBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides a wait step.
 *
 * @AutomationStep(
 *   id = "wait",
 *   label = @Translation("Wait"),
 *   description = @Translation("Wait for a condition or time"),
 *   category = "utility",
 *   icon = "wait",
 *   weight = 5,
 *   mcp_tool = "playwright_wait_for_selector"
 * )
 */
class WaitStep extends AutomationStepBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'wait_type' => 'time',
      'duration' => 1000,
      'selector' => '',
      'state' => 'visible',
      'timeout' => 30000,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['wait_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Wait type'),
      '#options' => [
        'time' => $this->t('Wait for time'),
        'element' => $this->t('Wait for element'),
      ],
      '#default_value' => $this->configuration['wait_type'],
      '#required' => TRUE,
    ];

    $form['duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Duration (ms)'),
      '#description' => $this->t('Time to wait in milliseconds.'),
      '#default_value' => $this->configuration['duration'],
      '#min' => 0,
      '#max' => 60000,
      '#step' => 100,
      '#states' => [
        'visible' => [
          ':input[name="wait_type"]' => ['value' => 'time'],
        ],
        'required' => [
          ':input[name="wait_type"]' => ['value' => 'time'],
        ],
      ],
    ];

    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element selector'),
      '#description' => $this->t('CSS selector or XPath to wait for.'),
      '#default_value' => $this->configuration['selector'],
      '#size' => 60,
      '#states' => [
        'visible' => [
          ':input[name="wait_type"]' => ['value' => 'element'],
        ],
        'required' => [
          ':input[name="wait_type"]' => ['value' => 'element'],
        ],
      ],
    ];

    $form['state'] = [
      '#type' => 'select',
      '#title' => $this->t('Element state'),
      '#options' => [
        'attached' => $this->t('Attached to DOM'),
        'detached' => $this->t('Detached from DOM'),
        'visible' => $this->t('Visible'),
        'hidden' => $this->t('Hidden'),
      ],
      '#default_value' => $this->configuration['state'],
      '#states' => [
        'visible' => [
          ':input[name="wait_type"]' => ['value' => 'element'],
        ],
      ],
    ];

    $form['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout (ms)'),
      '#description' => $this->t('Maximum time to wait for element.'),
      '#default_value' => $this->configuration['timeout'],
      '#min' => 0,
      '#max' => 300000,
      '#step' => 1000,
      '#states' => [
        'visible' => [
          ':input[name="wait_type"]' => ['value' => 'element'],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['wait_type'] = $form_state->getValue('wait_type');
    $this->configuration['duration'] = $form_state->getValue('duration');
    $this->configuration['selector'] = $form_state->getValue('selector');
    $this->configuration['state'] = $form_state->getValue('state');
    $this->configuration['timeout'] = $form_state->getValue('timeout');
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    
    if ($this->configuration['wait_type'] === 'time' && empty($this->configuration['duration'])) {
      $errors[] = $this->t('Duration is required for time wait.');
    }
    
    if ($this->configuration['wait_type'] === 'element' && empty($this->configuration['selector'])) {
      $errors[] = $this->t('Selector is required for element wait.');
    }
    
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function execute(array $context) {
    $mcp_client = $context['mcp_client'];
    $variables = $context['variables'] ?? [];
    
    if ($this->configuration['wait_type'] === 'time') {
      // Simple time wait.
      $duration = $this->configuration['duration'];
      $start = microtime(TRUE);
      usleep($duration * 1000); // Convert to microseconds.
      $actual_duration = (microtime(TRUE) - $start) * 1000;
      
      return [
        'success' => TRUE,
        'data' => [
          'wait_type' => 'time',
          'requested_duration' => $duration,
          'actual_duration' => round($actual_duration, 2),
        ],
        'duration' => round($actual_duration, 2),
      ];
    }
    
    // Element wait.
    $selector = $this->interpolateVariables($this->configuration['selector'], $variables);
    
    $measured = $this->measureExecution(function () use ($mcp_client, $selector) {
      return $mcp_client->execute([
        'tool' => 'playwright_wait_for_selector',
        'parameters' => [
          'selector' => $selector,
          'state' => $this->configuration['state'],
          'timeout' => $this->configuration['timeout'],
        ],
      ]);
    });
    
    $result = $measured['result'];
    
    return [
      'success' => !empty($result['success']),
      'data' => [
        'wait_type' => 'element',
        'selector' => $selector,
        'state' => $this->configuration['state'],
        'element_found' => !empty($result['data']['found']),
      ],
      'error' => $result['error'] ?? NULL,
      'duration' => $measured['duration'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function preview() {
    if ($this->configuration['wait_type'] === 'time') {
      return $this->t('Wait for @duration ms', ['@duration' => $this->configuration['duration']]);
    }
    
    return $this->t('Wait for @selector to be @state', [
      '@selector' => $this->configuration['selector'],
      '@state' => $this->configuration['state'],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getExamples() {
    return [
      [
        'label' => $this->t('Wait 2 seconds'),
        'config' => [
          'wait_type' => 'time',
          'duration' => 2000,
        ],
      ],
      [
        'label' => $this->t('Wait for element to appear'),
        'config' => [
          'wait_type' => 'element',
          'selector' => '.loading-complete',
          'state' => 'visible',
        ],
      ],
      [
        'label' => $this->t('Wait for spinner to disappear'),
        'config' => [
          'wait_type' => 'element',
          'selector' => '.spinner',
          'state' => 'hidden',
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getMcpTool() {
    return $this->configuration['wait_type'] === 'time' ? NULL : 'playwright_wait_for_selector';
  }

  /**
   * {@inheritdoc}
   */
  public function prepareMcpParameters(array $variables = []) {
    if ($this->configuration['wait_type'] === 'time') {
      return [];
    }
    
    return [
      'selector' => $this->interpolateVariables($this->configuration['selector'], $variables),
      'state' => $this->configuration['state'],
      'timeout' => $this->configuration['timeout'],
    ];
  }

}