<?php

namespace Drupal\ai_web_automation\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\AutomationStepManager;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'step_config_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "step_config_formatter",
 *   label = @Translation("Step configuration formatter"),
 *   field_types = {
 *     "step_config"
 *   }
 * )
 */
class StepConfigFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * Constructs a StepConfigFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\ai_web_automation\AutomationStepManager $step_manager
   *   The automation step manager.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    AutomationStepManager $step_manager
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->stepManager = $step_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('ai_web_automation.step_manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $value = $item->getValue();
      
      if (empty($value['type'])) {
        continue;
      }

      try {
        $step = $this->stepManager->createStepInstance($value['type'], $value['config'] ?? []);
        
        $elements[$delta] = [
          '#theme' => 'step_card',
          '#step' => $value,
          '#index' => $delta,
          '#cache' => [
            'tags' => $items->getEntity()->getCacheTags(),
          ],
        ];
        
        // Add step preview.
        $elements[$delta]['#step']['preview'] = $step->preview();
        
      } catch (\Exception $e) {
        $elements[$delta] = [
          '#markup' => $this->t('Invalid step configuration'),
        ];
      }
    }

    return $elements;
  }

}