<?php

namespace Drupal\ai_web_automation\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\TypedData\MapDataDefinition;

/**
 * Plugin implementation of the 'step_config' field type.
 *
 * @FieldType(
 *   id = "step_config",
 *   label = @Translation("Step configuration"),
 *   description = @Translation("Stores automation step configurations"),
 *   default_widget = "step_config_widget",
 *   default_formatter = "step_config_formatter"
 * )
 */
class StepConfigItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['type'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Step type'))
      ->setRequired(TRUE);

    $properties['weight'] = DataDefinition::create('integer')
      ->setLabel(new TranslatableMarkup('Weight'))
      ->setDescription(new TranslatableMarkup('The weight for ordering steps'));

    $properties['config'] = MapDataDefinition::create()
      ->setLabel(new TranslatableMarkup('Configuration'))
      ->setDescription(new TranslatableMarkup('Step-specific configuration'));

    $properties['conditions'] = MapDataDefinition::create()
      ->setLabel(new TranslatableMarkup('Conditions'))
      ->setDescription(new TranslatableMarkup('Conditional execution settings'));

    $properties['enabled'] = DataDefinition::create('boolean')
      ->setLabel(new TranslatableMarkup('Enabled'))
      ->setDescription(new TranslatableMarkup('Whether this step is enabled'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'type' => [
          'type' => 'varchar',
          'length' => 255,
          'not null' => TRUE,
        ],
        'weight' => [
          'type' => 'int',
          'not null' => TRUE,
          'default' => 0,
        ],
        'config' => [
          'type' => 'blob',
          'size' => 'normal',
          'serialize' => TRUE,
        ],
        'conditions' => [
          'type' => 'blob',
          'size' => 'normal',
          'serialize' => TRUE,
        ],
        'enabled' => [
          'type' => 'int',
          'size' => 'tiny',
          'not null' => TRUE,
          'default' => 1,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $type = $this->get('type')->getValue();
    return $type === NULL || $type === '';
  }

  /**
   * {@inheritdoc}
   */
  public function getValue() {
    $values = parent::getValue();
    
    // Ensure config and conditions are arrays.
    if (isset($values['config']) && is_string($values['config'])) {
      $values['config'] = unserialize($values['config']);
    }
    if (isset($values['conditions']) && is_string($values['conditions'])) {
      $values['conditions'] = unserialize($values['conditions']);
    }
    
    // Set defaults.
    $values += [
      'config' => [],
      'conditions' => [],
      'enabled' => TRUE,
      'weight' => 0,
    ];
    
    return $values;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    // Serialize config and conditions for storage.
    if (isset($values['config']) && is_array($values['config'])) {
      $values['config'] = serialize($values['config']);
    }
    if (isset($values['conditions']) && is_array($values['conditions'])) {
      $values['conditions'] = serialize($values['conditions']);
    }
    
    parent::setValue($values, $notify);
  }

}