<?php

namespace Drupal\ai_web_automation\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\ai_web_automation\AutomationStepManager;
use Drupal\Core\Form\SubformState;
use Drupal\Component\Utility\NestedArray;

/**
 * Plugin implementation of the 'step_config_widget' widget.
 *
 * @FieldWidget(
 *   id = "step_config_widget",
 *   label = @Translation("Step configuration"),
 *   field_types = {
 *     "step_config"
 *   }
 * )
 */
class StepConfigWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->stepManager = $container->get('ai_web_automation.step_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $item = $items[$delta];
    $value = $item->getValue();
    
    $parents = $element['#field_parents'];
    $field_name = $this->fieldDefinition->getName();
    $wrapper_id = 'step-config-wrapper-' . $field_name . '-' . $delta;
    
    $element['#type'] = 'fieldset';
    $element['#title'] = $this->t('Step @number', ['@number' => $delta + 1]);
    $element['#attributes']['id'] = $wrapper_id;
    
    // Step type selection.
    $element['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Step type'),
      '#options' => $this->stepManager->getStepOptions(),
      '#default_value' => $value['type'] ?? '',
      '#empty_option' => $this->t('- Select -'),
      '#required' => $element['#required'],
      '#ajax' => [
        'callback' => [get_class($this), 'updateStepConfig'],
        'wrapper' => $wrapper_id,
        'event' => 'change',
      ],
    ];
    
    // Weight for ordering.
    $element['weight'] = [
      '#type' => 'weight',
      '#title' => $this->t('Weight'),
      '#default_value' => $value['weight'] ?? 0,
      '#delta' => 50,
    ];
    
    // Enabled checkbox.
    $element['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#default_value' => $value['enabled'] ?? TRUE,
    ];
    
    // Step configuration container.
    $element['config'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['step-config-container']],
    ];
    
    // If a step type is selected, show its configuration form.
    $step_type = $form_state->getValue(array_merge($parents, [$field_name, $delta, 'type'])) ?? $value['type'] ?? '';
    
    if (!empty($step_type)) {
      try {
        $step = $this->stepManager->createStepInstance($step_type, $value['config'] ?? []);
        $subform_state = SubformState::createForSubform($element['config'], $form, $form_state);
        $element['config'] = $step->buildConfigurationForm($element['config'], $subform_state);
        
        // Add a preview of what the step will do.
        $element['preview'] = [
          '#type' => 'item',
          '#title' => $this->t('Preview'),
          '#markup' => '<em>' . $step->preview() . '</em>',
          '#prefix' => '<div class="step-preview">',
          '#suffix' => '</div>',
        ];
      }
      catch (\Exception $e) {
        $element['config']['error'] = [
          '#markup' => '<div class="messages messages--error">' . $this->t('Error loading step configuration: @error', ['@error' => $e->getMessage()]) . '</div>',
        ];
      }
    }
    
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $delta => &$value) {
      // Extract configuration from the subform.
      if (!empty($value['type']) && !empty($value['config'])) {
        try {
          $step = $this->stepManager->createStepInstance($value['type'], []);
          $subform = $value['config'];
          $subform_state = SubformState::createForSubform($subform, $form, $form_state);
          $step->submitConfigurationForm($subform, $subform_state);
          $value['config'] = $step->getConfiguration();
        }
        catch (\Exception $e) {
          // Keep the raw config if step processing fails.
        }
      }
      
      // Ensure conditions is an array.
      if (!isset($value['conditions'])) {
        $value['conditions'] = [];
      }
    }
    
    return $values;
  }

  /**
   * AJAX callback to update step configuration.
   */
  public static function updateStepConfig(array $form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $parents = array_slice($trigger['#array_parents'], 0, -1);
    $element = NestedArray::getValue($form, $parents);
    return $element;
  }

}