<?php

namespace Drupal\ai_web_automation\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\GuzzleException;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Service for communicating with Playwright MCP server.
 */
class PlaywrightMcpService {

  use StringTranslationTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Active browser sessions.
   *
   * @var array
   */
  protected $sessions = [];

  /**
   * Constructs a PlaywrightMcpService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory, ClientInterface $http_client) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->httpClient = $http_client;
    $this->logger = $logger_factory->get('ai_web_automation');
  }

  /**
   * Gets the MCP configuration.
   *
   * @return array
   *   The configuration array.
   */
  protected function getConfig() {
    $config = $this->configFactory->get('ai_web_automation.settings');
    return [
      'server_url' => $config->get('mcp_server_url'),
      'api_key' => $config->get('mcp_api_key'),
      'default_timeout' => $config->get('default_timeout'),
      'browser_context' => $config->get('browser_context'),
    ];
  }

  /**
   * Executes an MCP tool.
   *
   * @param array $request
   *   The request containing:
   *   - tool: The MCP tool name.
   *   - parameters: Tool parameters.
   *   - session_id: Optional session ID.
   *
   * @return array
   *   The response containing:
   *   - success: Boolean indicating success.
   *   - data: Response data.
   *   - error: Error message if failed.
   */
  public function execute(array $request) {
    $config = $this->getConfig();
    
    try {
      // Prepare the MCP request.
      $mcp_request = [
        'jsonrpc' => '2.0',
        'method' => 'tools/call',
        'params' => [
          'name' => $request['tool'],
          'arguments' => $request['parameters'] ?? [],
        ],
        'id' => uniqid('mcp_', TRUE),
      ];
      
      // Add session context if provided.
      if (!empty($request['session_id'])) {
        $mcp_request['params']['context'] = [
          'session_id' => $request['session_id'],
        ];
      }
      
      // Log the request.
      $this->logger->debug('MCP Request: @tool', [
        '@tool' => $request['tool'],
        'request' => $mcp_request,
      ]);
      
      // Send the request.
      $response = $this->httpClient->request('POST', $config['server_url'], [
        'json' => $mcp_request,
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $config['api_key'],
        ],
        'timeout' => ($request['parameters']['timeout'] ?? $config['default_timeout']) / 1000,
      ]);
      
      $body = json_decode($response->getBody()->getContents(), TRUE);
      
      // Check for JSON-RPC error.
      if (isset($body['error'])) {
        throw new \Exception($body['error']['message'] ?? 'Unknown MCP error');
      }
      
      // Process the result.
      $result = $body['result'] ?? [];
      
      // Handle screenshot data.
      if (isset($result['screenshot'])) {
        $result['file_id'] = $this->saveScreenshot($result['screenshot']);
      }
      
      return [
        'success' => TRUE,
        'data' => $result,
        'error' => NULL,
      ];
      
    } catch (GuzzleException $e) {
      $this->logger->error('MCP HTTP error: @message', ['@message' => $e->getMessage()]);
      return [
        'success' => FALSE,
        'data' => NULL,
        'error' => $this->t('Failed to connect to MCP server: @message', ['@message' => $e->getMessage()]),
      ];
    } catch (\Exception $e) {
      $this->logger->error('MCP error: @message', ['@message' => $e->getMessage()]);
      return [
        'success' => FALSE,
        'data' => NULL,
        'error' => $e->getMessage(),
      ];
    }
  }

  /**
   * Creates a new browser session.
   *
   * @param array $options
   *   Session options.
   *
   * @return string|null
   *   The session ID or NULL on failure.
   */
  public function createSession(array $options = []) {
    $config = $this->getConfig();
    $browser_config = $config['browser_context'];
    
    $result = $this->execute([
      'tool' => 'playwright_launch',
      'parameters' => [
        'headless' => $browser_config['headless'] ?? TRUE,
        'viewport' => $browser_config['viewport'] ?? ['width' => 1280, 'height' => 720],
      ] + $options,
    ]);
    
    if ($result['success'] && !empty($result['data']['session_id'])) {
      $session_id = $result['data']['session_id'];
      $this->sessions[$session_id] = [
        'created' => time(),
        'options' => $options,
      ];
      return $session_id;
    }
    
    return NULL;
  }

  /**
   * Closes a browser session.
   *
   * @param string $session_id
   *   The session ID.
   *
   * @return bool
   *   TRUE if successful.
   */
  public function closeSession($session_id) {
    $result = $this->execute([
      'tool' => 'playwright_close',
      'parameters' => [],
      'session_id' => $session_id,
    ]);
    
    if ($result['success']) {
      unset($this->sessions[$session_id]);
      return TRUE;
    }
    
    return FALSE;
  }

  /**
   * Gets all active sessions.
   *
   * @return array
   *   Array of session information.
   */
  public function getActiveSessions() {
    return $this->sessions;
  }

  /**
   * Saves screenshot data to a file.
   *
   * @param string $data
   *   Base64 encoded screenshot data.
   *
   * @return int|null
   *   The file ID or NULL on failure.
   */
  protected function saveScreenshot($data) {
    try {
      // Decode base64 data.
      $binary = base64_decode($data);
      
      // Generate filename.
      $filename = 'automation_screenshot_' . time() . '_' . uniqid() . '.jpg';
      $uri = 'private://ai_web_automation/screenshots/' . $filename;
      
      // Ensure directory exists.
      $directory = dirname($uri);
      \Drupal::service('file_system')->prepareDirectory($directory, \Drupal\Core\File\FileSystemInterface::CREATE_DIRECTORY);
      
      // Save the file.
      $file = \Drupal::service('file.repository')->writeData($binary, $uri, \Drupal\Core\File\FileSystemInterface::EXISTS_REPLACE);
      
      if ($file) {
        return $file->id();
      }
    } catch (\Exception $e) {
      $this->logger->error('Failed to save screenshot: @message', ['@message' => $e->getMessage()]);
    }
    
    return NULL;
  }

  /**
   * Tests the MCP connection.
   *
   * @return array
   *   Test result with 'success' and 'message' keys.
   */
  public function testConnection() {
    try {
      $result = $this->execute([
        'tool' => 'playwright_version',
        'parameters' => [],
      ]);
      
      if ($result['success']) {
        return [
          'success' => TRUE,
          'message' => $this->t('Successfully connected to MCP server. Playwright version: @version', [
            '@version' => $result['data']['version'] ?? 'Unknown',
          ]),
        ];
      }
      
      return [
        'success' => FALSE,
        'message' => $result['error'] ?? $this->t('Failed to connect to MCP server.'),
      ];
      
    } catch (\Exception $e) {
      return [
        'success' => FALSE,
        'message' => $this->t('Connection error: @message', ['@message' => $e->getMessage()]),
      ];
    }
  }

  /**
   * Validates URL against security settings.
   *
   * @param string $url
   *   The URL to validate.
   *
   * @return bool
   *   TRUE if URL is allowed.
   */
  public function isUrlAllowed($url) {
    $config = $this->configFactory->get('ai_web_automation.settings');
    $whitelist = $config->get('security.url_whitelist') ?? [];
    $blacklist = $config->get('security.url_blacklist') ?? [];
    
    // Check blacklist first.
    foreach ($blacklist as $pattern) {
      if (fnmatch($pattern, $url)) {
        return FALSE;
      }
    }
    
    // If whitelist is empty, allow all non-blacklisted URLs.
    if (empty($whitelist)) {
      return TRUE;
    }
    
    // Check whitelist.
    foreach ($whitelist as $pattern) {
      if (fnmatch($pattern, $url)) {
        return TRUE;
      }
    }
    
    return FALSE;
  }

}