<?php

namespace Drupal\ai_web_automation\Service;

use Drupal\Core\Database\Connection;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Service for storing and retrieving automation execution results.
 */
class ResultsStorage {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a ResultsStorage object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(Connection $database, FileSystemInterface $file_system, EntityTypeManagerInterface $entity_type_manager) {
    $this->database = $database;
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Initializes a new execution record.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param int $scenario_id
   *   The scenario entity ID.
   * @param array $options
   *   Execution options.
   *
   * @return bool
   *   TRUE if successful.
   */
  public function initializeExecution($execution_id, $scenario_id, array $options = []) {
    try {
      $this->database->insert('ai_web_automation_results')
        ->fields([
          'execution_id' => $execution_id,
          'scenario_id' => $scenario_id,
          'uid' => \Drupal::currentUser()->id(),
          'status' => 'running',
          'started' => time(),
          'step_results' => serialize([]),
          'performance_metrics' => serialize([
            'options' => $options,
            'start_time' => microtime(TRUE),
          ]),
        ])
        ->execute();
      
      return TRUE;
    } catch (\Exception $e) {
      \Drupal::logger('ai_web_automation')->error('Failed to initialize execution: @message', ['@message' => $e->getMessage()]);
      return FALSE;
    }
  }

  /**
   * Updates the execution progress.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param int $current_step
   *   The current step index.
   * @param int $total_steps
   *   The total number of steps.
   */
  public function updateExecutionProgress($execution_id, $current_step, $total_steps) {
    try {
      $metrics = $this->getPerformanceMetrics($execution_id);
      $metrics['current_step'] = $current_step;
      $metrics['total_steps'] = $total_steps;
      $metrics['progress'] = round(($current_step / $total_steps) * 100);
      
      $this->database->update('ai_web_automation_results')
        ->fields([
          'performance_metrics' => serialize($metrics),
        ])
        ->condition('execution_id', $execution_id)
        ->execute();
    } catch (\Exception $e) {
      \Drupal::logger('ai_web_automation')->error('Failed to update progress: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * Adds a step result to the execution.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param int $step_index
   *   The step index.
   * @param array $result
   *   The step result.
   */
  public function addStepResult($execution_id, $step_index, array $result) {
    try {
      $step_results = $this->getStepResults($execution_id);
      $step_results[$step_index] = $result + [
        'timestamp' => time(),
        'index' => $step_index,
      ];
      
      $this->database->update('ai_web_automation_results')
        ->fields([
          'step_results' => serialize($step_results),
        ])
        ->condition('execution_id', $execution_id)
        ->execute();
    } catch (\Exception $e) {
      \Drupal::logger('ai_web_automation')->error('Failed to add step result: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * Completes an execution.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param bool $success
   *   Whether the execution was successful.
   * @param array $step_results
   *   All step results.
   */
  public function completeExecution($execution_id, $success, array $step_results) {
    try {
      $metrics = $this->getPerformanceMetrics($execution_id);
      $metrics['end_time'] = microtime(TRUE);
      $metrics['total_duration'] = round(($metrics['end_time'] - $metrics['start_time']) * 1000, 2);
      $metrics['success_rate'] = $this->calculateSuccessRate($step_results);
      
      $screenshots = [];
      foreach ($step_results as $result) {
        if (!empty($result['screenshot'])) {
          $screenshots[] = $result['screenshot'];
        }
      }
      
      $this->database->update('ai_web_automation_results')
        ->fields([
          'status' => $success ? 'completed' : 'failed',
          'completed' => time(),
          'step_results' => serialize($step_results),
          'screenshots' => serialize($screenshots),
          'performance_metrics' => serialize($metrics),
        ])
        ->condition('execution_id', $execution_id)
        ->execute();
    } catch (\Exception $e) {
      \Drupal::logger('ai_web_automation')->error('Failed to complete execution: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * Marks an execution as failed.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param string $error_message
   *   The error message.
   */
  public function markExecutionFailed($execution_id, $error_message) {
    try {
      $this->database->update('ai_web_automation_results')
        ->fields([
          'status' => 'failed',
          'completed' => time(),
          'error_message' => $error_message,
        ])
        ->condition('execution_id', $execution_id)
        ->execute();
    } catch (\Exception $e) {
      \Drupal::logger('ai_web_automation')->error('Failed to mark execution as failed: @message', ['@message' => $e->getMessage()]);
    }
  }

  /**
   * Gets execution results.
   *
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array|null
   *   The execution results or NULL if not found.
   */
  public function getExecutionResults($execution_id) {
    $result = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r')
      ->condition('execution_id', $execution_id)
      ->execute()
      ->fetchAssoc();
    
    if ($result) {
      $result['step_results'] = unserialize($result['step_results']);
      $result['screenshots'] = unserialize($result['screenshots']);
      $result['performance_metrics'] = unserialize($result['performance_metrics']);
      return $result;
    }
    
    return NULL;
  }

  /**
   * Gets execution status.
   *
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array|null
   *   The execution status or NULL if not found.
   */
  public function getExecutionStatus($execution_id) {
    $result = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r', ['status', 'started', 'completed'])
      ->condition('execution_id', $execution_id)
      ->execute()
      ->fetchAssoc();
    
    if ($result) {
      $metrics = $this->getPerformanceMetrics($execution_id);
      $result['progress'] = $metrics['progress'] ?? 0;
      $result['current_step'] = $metrics['current_step'] ?? 0;
      $result['total_steps'] = $metrics['total_steps'] ?? 0;
      return $result;
    }
    
    return NULL;
  }

  /**
   * Gets executions for a scenario.
   *
   * @param int $scenario_id
   *   The scenario ID.
   * @param int $limit
   *   Maximum number of results.
   *
   * @return array
   *   Array of execution records.
   */
  public function getScenarioExecutions($scenario_id, $limit = 10) {
    $results = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r', ['id', 'execution_id', 'uid', 'status', 'started', 'completed'])
      ->condition('scenario_id', $scenario_id)
      ->orderBy('started', 'DESC')
      ->range(0, $limit)
      ->execute()
      ->fetchAll();
    
    return $results;
  }

  /**
   * Cleans up old execution results.
   *
   * @param int $days
   *   Number of days to retain results.
   *
   * @return int
   *   Number of records deleted.
   */
  public function cleanupOldResults($days = 30) {
    $cutoff = time() - ($days * 86400);
    
    // Get executions to delete.
    $executions = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r', ['id', 'screenshots'])
      ->condition('started', $cutoff, '<')
      ->execute()
      ->fetchAll();
    
    $count = 0;
    foreach ($executions as $execution) {
      // Delete associated screenshots.
      $screenshots = unserialize($execution->screenshots);
      if (!empty($screenshots)) {
        $file_storage = $this->entityTypeManager->getStorage('file');
        $files = $file_storage->loadMultiple($screenshots);
        $file_storage->delete($files);
      }
      
      // Delete the execution record.
      $this->database->delete('ai_web_automation_results')
        ->condition('id', $execution->id)
        ->execute();
      
      $count++;
    }
    
    return $count;
  }

  /**
   * Gets performance metrics for an execution.
   *
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array
   *   The performance metrics.
   */
  protected function getPerformanceMetrics($execution_id) {
    $result = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r', ['performance_metrics'])
      ->condition('execution_id', $execution_id)
      ->execute()
      ->fetchField();
    
    return $result ? unserialize($result) : [];
  }

  /**
   * Gets step results for an execution.
   *
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array
   *   The step results.
   */
  protected function getStepResults($execution_id) {
    $result = $this->database->select('ai_web_automation_results', 'r')
      ->fields('r', ['step_results'])
      ->condition('execution_id', $execution_id)
      ->execute()
      ->fetchField();
    
    return $result ? unserialize($result) : [];
  }

  /**
   * Calculates the success rate of steps.
   *
   * @param array $step_results
   *   The step results.
   *
   * @return float
   *   The success rate as a percentage.
   */
  protected function calculateSuccessRate(array $step_results) {
    if (empty($step_results)) {
      return 0;
    }
    
    $successful = 0;
    foreach ($step_results as $result) {
      if (!empty($result['success'])) {
        $successful++;
      }
    }
    
    return round(($successful / count($step_results)) * 100, 2);
  }

}