<?php

namespace Drupal\ai_web_automation\Service;

use Drupal\ai_web_automation\Entity\AutomationScenarioInterface;
use Drupal\ai_web_automation\AutomationStepManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Service for executing automation scenarios.
 */
class ScenarioExecutor {

  /**
   * The MCP client service.
   *
   * @var \Drupal\ai_web_automation\Service\PlaywrightMcpService
   */
  protected $mcpClient;

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * The results storage service.
   *
   * @var \Drupal\ai_web_automation\Service\ResultsStorage
   */
  protected $resultsStorage;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a ScenarioExecutor object.
   *
   * @param \Drupal\ai_web_automation\Service\PlaywrightMcpService $mcp_client
   *   The MCP client service.
   * @param \Drupal\ai_web_automation\AutomationStepManager $step_manager
   *   The automation step manager.
   * @param \Drupal\ai_web_automation\Service\ResultsStorage $results_storage
   *   The results storage service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    PlaywrightMcpService $mcp_client,
    AutomationStepManager $step_manager,
    ResultsStorage $results_storage,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->mcpClient = $mcp_client;
    $this->stepManager = $step_manager;
    $this->resultsStorage = $results_storage;
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
    $this->logger = $logger_factory->get('ai_web_automation');
  }

  /**
   * Executes an automation scenario.
   *
   * @param \Drupal\ai_web_automation\Entity\AutomationScenarioInterface $scenario
   *   The scenario to execute.
   * @param array $options
   *   Execution options:
   *   - mode: 'normal', 'dry_run', or 'step_by_step'.
   *   - capture_screenshots: Boolean.
   *   - stop_on_error: Boolean.
   *   - headless: Boolean.
   *   - variables: Array of variable overrides.
   *   - live_view: Boolean for live execution view.
   *
   * @return string|null
   *   The execution ID or NULL on failure.
   */
  public function execute(AutomationScenarioInterface $scenario, array $options = []) {
    // Generate execution ID.
    $execution_id = uniqid('exec_' . $scenario->id() . '_', TRUE);
    
    // Initialize execution record.
    $this->resultsStorage->initializeExecution($execution_id, $scenario->id(), $options);
    
    // Log execution start.
    $this->logger->info('Starting execution of scenario @name (ID: @id)', [
      '@name' => $scenario->getName(),
      '@id' => $execution_id,
    ]);
    
    // Check if scenario is published.
    if (!$scenario->isPublished()) {
      $this->resultsStorage->markExecutionFailed($execution_id, 'Scenario is not published.');
      return NULL;
    }
    
    // Get steps.
    $steps = $scenario->getSteps();
    if (empty($steps)) {
      $this->resultsStorage->markExecutionFailed($execution_id, 'No steps defined in scenario.');
      return NULL;
    }
    
    // Merge variables.
    $variables = array_merge($scenario->getVariables(), $options['variables'] ?? []);
    
    // Create browser session if not dry run.
    $session_id = NULL;
    if ($options['mode'] !== 'dry_run') {
      $session_id = $this->mcpClient->createSession([
        'headless' => $options['headless'] ?? TRUE,
      ]);
      
      if (!$session_id) {
        $this->resultsStorage->markExecutionFailed($execution_id, 'Failed to create browser session.');
        return NULL;
      }
    }
    
    // Execute steps.
    $context = [
      'mcp_client' => $this->mcpClient,
      'variables' => $variables,
      'previous_results' => [],
      'session_id' => $session_id,
      'execution_id' => $execution_id,
      'options' => $options,
    ];
    
    $step_results = [];
    $overall_success = TRUE;
    
    foreach ($steps as $index => $step_config) {
      // Skip disabled steps.
      if (empty($step_config['enabled'])) {
        continue;
      }
      
      // Create step instance.
      try {
        $step = $this->stepManager->createStepInstance($step_config['type'], $step_config['config'] ?? []);
      } catch (\Exception $e) {
        $step_results[$index] = [
          'success' => FALSE,
          'error' => 'Failed to create step: ' . $e->getMessage(),
          'duration' => 0,
        ];
        $overall_success = FALSE;
        
        if ($options['stop_on_error']) {
          break;
        }
        continue;
      }
      
      // Update execution status.
      $this->resultsStorage->updateExecutionProgress($execution_id, $index, count($steps));
      
      // Execute step.
      if ($options['mode'] === 'dry_run') {
        // Dry run - just preview.
        $step_results[$index] = [
          'success' => TRUE,
          'preview' => $step->preview(),
          'duration' => 0,
        ];
      } else {
        // Normal execution.
        try {
          // Capture screenshot before step if requested.
          if ($options['capture_screenshots'] && $index === 0) {
            $this->captureScreenshot($context, 'before_execution');
          }
          
          $result = $step->execute($context);
          $step_results[$index] = $result;
          
          // Update context with results.
          $context['previous_results'][$index] = $result;
          
          // Update variables if step extracted data.
          if (!empty($result['data']['variable_name']) && isset($result['data']['value'])) {
            $context['variables'][$result['data']['variable_name']] = $result['data']['value'];
          }
          
          // Capture screenshot after step if requested.
          if ($options['capture_screenshots'] && !$step->capturesScreenshot()) {
            $screenshot_id = $this->captureScreenshot($context, 'step_' . $index);
            $step_results[$index]['screenshot'] = $screenshot_id;
          }
          
          if (!$result['success']) {
            $overall_success = FALSE;
            if ($options['stop_on_error']) {
              break;
            }
          }
          
        } catch (\Exception $e) {
          $step_results[$index] = [
            'success' => FALSE,
            'error' => 'Step execution failed: ' . $e->getMessage(),
            'duration' => 0,
          ];
          $overall_success = FALSE;
          
          if ($options['stop_on_error']) {
            break;
          }
        }
      }
      
      // Store step result.
      $this->resultsStorage->addStepResult($execution_id, $index, $step_results[$index]);
      
      // Pause for step-by-step mode.
      if ($options['mode'] === 'step_by_step' && $index < count($steps) - 1) {
        // In a real implementation, this would pause and wait for user input.
        // For now, we'll just add a small delay.
        sleep(1);
      }
    }
    
    // Close browser session.
    if ($session_id) {
      $this->mcpClient->closeSession($session_id);
    }
    
    // Complete execution.
    $this->resultsStorage->completeExecution($execution_id, $overall_success, $step_results);
    
    // Update scenario statistics.
    $scenario->setLastExecutionTime(time());
    $scenario->incrementExecutionCount();
    $scenario->save();
    
    return $execution_id;
  }

  /**
   * Executes a single step for live execution.
   *
   * @param string $execution_id
   *   The execution ID.
   * @param int $step_index
   *   The step index.
   * @param array $context
   *   The execution context.
   *
   * @return array
   *   The step result.
   */
  public function executeStep($execution_id, $step_index, array $context) {
    // This method would be called via AJAX for live execution.
    // Implementation would be similar to the step execution in execute().
    return [];
  }

  /**
   * Captures a screenshot.
   *
   * @param array $context
   *   The execution context.
   * @param string $name
   *   The screenshot name.
   *
   * @return int|null
   *   The file ID or NULL.
   */
  protected function captureScreenshot(array $context, $name) {
    $result = $context['mcp_client']->execute([
      'tool' => 'playwright_screenshot',
      'parameters' => [
        'type' => 'jpeg',
        'quality' => 80,
        'fullPage' => FALSE,
      ],
      'session_id' => $context['session_id'],
    ]);
    
    if (!empty($result['success']) && !empty($result['data']['file_id'])) {
      return $result['data']['file_id'];
    }
    
    return NULL;
  }

  /**
   * Gets the status of an ongoing execution.
   *
   * @param string $execution_id
   *   The execution ID.
   *
   * @return array|null
   *   The execution status or NULL if not found.
   */
  public function getExecutionStatus($execution_id) {
    return $this->resultsStorage->getExecutionStatus($execution_id);
  }

}