<?php

namespace Drupal\ai_web_automation\Service;

use Drupal\ai_web_automation\AutomationStepManager;

/**
 * Service for validating automation steps.
 */
class StepValidator {

  /**
   * The automation step manager.
   *
   * @var \Drupal\ai_web_automation\AutomationStepManager
   */
  protected $stepManager;

  /**
   * Constructs a StepValidator object.
   *
   * @param \Drupal\ai_web_automation\AutomationStepManager $step_manager
   *   The automation step manager.
   */
  public function __construct(AutomationStepManager $step_manager) {
    $this->stepManager = $step_manager;
  }

  /**
   * Validates all steps in a scenario.
   *
   * @param array $steps
   *   Array of step configurations.
   *
   * @return array
   *   Array of validation results, each containing:
   *   - valid: Boolean indicating if the step is valid.
   *   - errors: Array of error messages.
   */
  public function validateScenarioSteps(array $steps) {
    $results = [];
    
    foreach ($steps as $index => $step_config) {
      if (empty($step_config['type'])) {
        $results[$index] = [
          'valid' => FALSE,
          'errors' => ['Step type is required.'],
        ];
        continue;
      }
      
      $errors = $this->stepManager->validateStep($step_config['type'], $step_config['config'] ?? []);
      $results[$index] = [
        'valid' => empty($errors),
        'errors' => $errors,
      ];
    }
    
    return $results;
  }

  /**
   * Validates step dependencies and conditions.
   *
   * @param array $steps
   *   Array of step configurations.
   *
   * @return array
   *   Array of validation errors.
   */
  public function validateStepDependencies(array $steps) {
    $errors = [];
    $available_variables = [];
    
    foreach ($steps as $index => $step_config) {
      if (empty($step_config['type'])) {
        continue;
      }
      
      // Check if step uses variables that haven't been defined yet.
      $used_variables = $this->extractUsedVariables($step_config['config'] ?? []);
      foreach ($used_variables as $var) {
        if (!in_array($var, $available_variables)) {
          $errors[] = sprintf('Step %d uses undefined variable: {{%s}}', $index + 1, $var);
        }
      }
      
      // If this is an extract step, add its variable to available variables.
      if ($step_config['type'] === 'extract' && !empty($step_config['config']['variable_name'])) {
        $available_variables[] = $step_config['config']['variable_name'];
      }
      
      // Validate conditions.
      if (!empty($step_config['conditions'])) {
        $condition_errors = $this->validateConditions($step_config['conditions'], $available_variables);
        foreach ($condition_errors as $error) {
          $errors[] = sprintf('Step %d condition error: %s', $index + 1, $error);
        }
      }
    }
    
    return $errors;
  }

  /**
   * Extracts variable names used in a configuration.
   *
   * @param array $config
   *   The configuration array.
   *
   * @return array
   *   Array of variable names.
   */
  protected function extractUsedVariables(array $config) {
    $variables = [];
    
    array_walk_recursive($config, function ($value) use (&$variables) {
      if (is_string($value)) {
        preg_match_all('/\{\{(\w+)\}\}/', $value, $matches);
        if (!empty($matches[1])) {
          $variables = array_merge($variables, $matches[1]);
        }
      }
    });
    
    return array_unique($variables);
  }

  /**
   * Validates step conditions.
   *
   * @param array $conditions
   *   The conditions configuration.
   * @param array $available_variables
   *   Available variables at this point.
   *
   * @return array
   *   Array of validation errors.
   */
  protected function validateConditions(array $conditions, array $available_variables) {
    $errors = [];
    
    // This is a placeholder for more complex condition validation.
    // In a full implementation, this would validate condition syntax,
    // variable references, and logical operators.
    
    return $errors;
  }

  /**
   * Validates URL patterns against security settings.
   *
   * @param array $steps
   *   Array of step configurations.
   * @param array $whitelist
   *   URL whitelist patterns.
   * @param array $blacklist
   *   URL blacklist patterns.
   *
   * @return array
   *   Array of validation errors.
   */
  public function validateUrlSecurity(array $steps, array $whitelist, array $blacklist) {
    $errors = [];
    
    foreach ($steps as $index => $step_config) {
      if ($step_config['type'] === 'navigate' && !empty($step_config['config']['url'])) {
        $url = $step_config['config']['url'];
        
        // Skip variable placeholders.
        if (strpos($url, '{{') !== FALSE) {
          continue;
        }
        
        // Check blacklist.
        foreach ($blacklist as $pattern) {
          if (fnmatch($pattern, $url)) {
            $errors[] = sprintf('Step %d: URL "%s" matches blacklist pattern "%s"', $index + 1, $url, $pattern);
          }
        }
        
        // Check whitelist if not empty.
        if (!empty($whitelist)) {
          $allowed = FALSE;
          foreach ($whitelist as $pattern) {
            if (fnmatch($pattern, $url)) {
              $allowed = TRUE;
              break;
            }
          }
          if (!$allowed) {
            $errors[] = sprintf('Step %d: URL "%s" does not match any whitelist pattern', $index + 1, $url);
          }
        }
      }
    }
    
    return $errors;
  }

}